package com.google.appinventor.components.runtime.util;

import android.app.Activity;
import android.content.res.ColorStateList;
import android.graphics.PorterDuff;
import android.support.v7.internal.widget.TintImageView;
import android.view.View;
import android.view.ViewGroup;
import java.util.LinkedList;
import java.util.Queue;
/* loaded from: classes.dex */
public final class ImageViewUtil {
    private static final boolean canUpdate;

    static {
        boolean updateable = false;
        try {
            TintImageView.class.getMethod("setImageTintMode", PorterDuff.Mode.class);
            updateable = true;
        } catch (NoSuchMethodException e) {
        }
        canUpdate = updateable;
    }

    private ImageViewUtil() {
    }

    public static void setMenuButtonColor(Activity activity, int color) {
        TintImageView overflowMenuView;
        if (canUpdate && (overflowMenuView = findOverflowMenuView(activity)) != null) {
            ColorStateList stateList = new ColorStateList(new int[][]{new int[0]}, new int[]{color});
            try {
                overflowMenuView.setImageTintMode(PorterDuff.Mode.MULTIPLY);
                overflowMenuView.setImageTintList(stateList);
            } catch (NoSuchMethodError e) {
            }
        }
    }

    private static TintImageView findOverflowMenuView(Activity activity) {
        ViewGroup vg = (ViewGroup) activity.getWindow().getDecorView();
        Queue<ViewGroup> children = new LinkedList<>();
        children.add(vg);
        while (children.size() > 0) {
            ViewGroup vg2 = children.poll();
            ViewGroup vg3 = vg2;
            for (int i = 0; i < vg3.getChildCount(); i++) {
                View child = vg3.getChildAt(i);
                if (child instanceof TintImageView) {
                    TintImageView overflowMenuView = (TintImageView) child;
                    return overflowMenuView;
                }
                if (child instanceof ViewGroup) {
                    children.add((ViewGroup) child);
                }
            }
        }
        return null;
    }
}
