package gnu.bytecode;
/* loaded from: classes.dex */
public class PrimType extends Type {
    private static final String numberHierarchy = "A:java.lang.Byte;B:java.lang.Short;C:java.lang.Integer;D:java.lang.Long;E:gnu.math.IntNum;E:java.gnu.math.BitInteger;G:gnu.math.RatNum;H:java.lang.Float;I:java.lang.Double;I:gnu.math.DFloNum;J:gnu.math.RealNum;K:gnu.math.Complex;L:gnu.math.Quantity;K:gnu.math.Numeric;N:java.lang.Number;";

    public PrimType(String nam, String sig, int siz, Class reflectClass) {
        super(nam, sig);
        this.size = siz;
        this.reflectClass = reflectClass;
        Type.registerTypeForClass(reflectClass, this);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public PrimType(PrimType type) {
        super(type.this_name, type.signature);
        this.size = type.size;
        this.reflectClass = type.reflectClass;
    }

    @Override // gnu.bytecode.Type
    public Object coerceFromObject(Object obj) {
        if (obj.getClass() != this.reflectClass) {
            char sig1 = (this.signature == null || this.signature.length() != 1) ? ' ' : this.signature.charAt(0);
            switch (sig1) {
                case 'B':
                    return Byte.valueOf(((Number) obj).byteValue());
                case 'D':
                    return Double.valueOf(((Number) obj).doubleValue());
                case 'F':
                    return Float.valueOf(((Number) obj).floatValue());
                case 'I':
                    return Integer.valueOf(((Number) obj).intValue());
                case 'J':
                    return Long.valueOf(((Number) obj).longValue());
                case 'S':
                    return Short.valueOf(((Number) obj).shortValue());
                case 'Z':
                    return Boolean.valueOf(((Boolean) obj).booleanValue());
                default:
                    throw new ClassCastException("don't know how to coerce " + obj.getClass().getName() + " to " + getName());
            }
        }
        return obj;
    }

    public char charValue(Object value) {
        return ((Character) value).charValue();
    }

    public static boolean booleanValue(Object value) {
        return !(value instanceof Boolean) || ((Boolean) value).booleanValue();
    }

    public ClassType boxedType() {
        String cname;
        char sig1 = getSignature().charAt(0);
        switch (sig1) {
            case 'B':
                cname = "java.lang.Byte";
                break;
            case 'C':
                cname = "java.lang.Character";
                break;
            case 'D':
                cname = "java.lang.Double";
                break;
            case 'F':
                cname = "java.lang.Float";
                break;
            case 'I':
                cname = "java.lang.Integer";
                break;
            case 'J':
                cname = "java.lang.Long";
                break;
            case 'S':
                cname = "java.lang.Short";
                break;
            case 'Z':
                cname = "java.lang.Boolean";
                break;
            default:
                cname = null;
                break;
        }
        return ClassType.make(cname);
    }

    @Override // gnu.bytecode.Type
    public void emitCoerceToObject(CodeAttr code) {
        Method method;
        char sig1 = getSignature().charAt(0);
        ClassType clas = boxedType();
        if (sig1 == 'Z') {
            code.emitIfIntNotZero();
            code.emitGetStatic(clas.getDeclaredField("TRUE"));
            code.emitElse();
            code.emitGetStatic(clas.getDeclaredField("FALSE"));
            code.emitFi();
            return;
        }
        Type[] args = {this};
        if (code.getMethod().getDeclaringClass().classfileFormatVersion >= 3211264) {
            method = clas.getDeclaredMethod("valueOf", args);
        } else {
            method = clas.getDeclaredMethod("<init>", args);
            code.emitNew(clas);
            code.emitDupX();
            code.emitSwap();
        }
        code.emitInvoke(method);
    }

    @Override // gnu.bytecode.Type
    public void emitIsInstance(CodeAttr code) {
        char sig1 = (this.signature == null || this.signature.length() != 1) ? ' ' : this.signature.charAt(0);
        if (sig1 == 'Z') {
            javalangBooleanType.emitIsInstance(code);
        } else if (sig1 == 'V') {
            code.emitPop(1);
            code.emitPushInt(1);
        } else {
            javalangNumberType.emitIsInstance(code);
        }
    }

    @Override // gnu.bytecode.Type
    public void emitCoerceFromObject(CodeAttr code) {
        char sig1 = (this.signature == null || this.signature.length() != 1) ? ' ' : this.signature.charAt(0);
        if (sig1 == 'Z') {
            code.emitCheckcast(javalangBooleanType);
            code.emitInvokeVirtual(booleanValue_method);
        } else if (sig1 == 'V') {
            code.emitPop(1);
        } else {
            code.emitCheckcast(javalangNumberType);
            if (sig1 == 'I' || sig1 == 'S' || sig1 == 'B') {
                code.emitInvokeVirtual(intValue_method);
            } else if (sig1 == 'J') {
                code.emitInvokeVirtual(longValue_method);
            } else if (sig1 == 'D') {
                code.emitInvokeVirtual(doubleValue_method);
            } else if (sig1 == 'F') {
                code.emitInvokeVirtual(floatValue_method);
            } else {
                super.emitCoerceFromObject(code);
            }
        }
    }

    public static int compare(PrimType type1, PrimType type2) {
        char sig1 = type1.signature.charAt(0);
        char sig2 = type2.signature.charAt(0);
        if (sig1 == sig2) {
            return 0;
        }
        if (sig1 == 'V') {
            return 1;
        }
        if (sig2 != 'V') {
            if (sig1 == 'Z' || sig2 == 'Z') {
                return -3;
            }
            if (sig1 == 'C') {
                return type2.size <= 2 ? -3 : -1;
            } else if (sig2 == 'C') {
                return type1.size > 2 ? 1 : -3;
            } else if (sig1 == 'D') {
                return 1;
            } else {
                if (sig2 != 'D') {
                    if (sig1 == 'F') {
                        return 1;
                    }
                    if (sig2 != 'F') {
                        if (sig1 == 'J') {
                            return 1;
                        }
                        if (sig2 != 'J') {
                            if (sig1 == 'I') {
                                return 1;
                            }
                            if (sig2 != 'I') {
                                if (sig1 == 'S') {
                                    return 1;
                                }
                                return sig2 != 'S' ? -3 : -1;
                            }
                            return -1;
                        }
                        return -1;
                    }
                    return -1;
                }
                return -1;
            }
        }
        return -1;
    }

    public Type promotedType() {
        switch (this.signature.charAt(0)) {
            case 'B':
            case 'C':
            case 'I':
            case 'S':
            case 'Z':
                return Type.intType;
            default:
                return getImplementationType();
        }
    }

    private static char findInHierarchy(String cname) {
        int pos = numberHierarchy.indexOf(cname) - 2;
        if (pos < 0) {
            return (char) 0;
        }
        return numberHierarchy.charAt(pos);
    }

    /* JADX WARN: Removed duplicated region for block: B:21:0x0045  */
    /* JADX WARN: Removed duplicated region for block: B:41:0x0080  */
    /* JADX WARN: Removed duplicated region for block: B:56:? A[RETURN, SYNTHETIC] */
    @Override // gnu.bytecode.Type
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public int compare(Type other) {
        char otherPriority;
        if (other instanceof PrimType) {
            if (other.getImplementationType() != other) {
                return swappedCompareResult(other.compare(this));
            }
            return compare(this, (PrimType) other);
        } else if (!(other instanceof ClassType)) {
            if (other instanceof ArrayType) {
                return -3;
            }
            return swappedCompareResult(other.compare(this));
        } else {
            char sig1 = this.signature.charAt(0);
            String otherName = other.getName();
            if (otherName == null) {
                return -1;
            }
            char thisPriority = 0;
            switch (sig1) {
                case 'B':
                    thisPriority = 'A';
                    if (thisPriority != 0 || (otherPriority = findInHierarchy(otherName)) == 0) {
                        return (!otherName.equals("java.lang.Object") || other == toStringType) ? -1 : -3;
                    } else if (otherPriority != thisPriority) {
                        return otherPriority < thisPriority ? 1 : -1;
                    } else {
                        return 0;
                    }
                case 'C':
                    if (otherName.equals("java.lang.Character")) {
                        return 0;
                    }
                    if (thisPriority != 0) {
                        break;
                    }
                    if (otherName.equals("java.lang.Object")) {
                        break;
                    }
                case 'D':
                    thisPriority = Access.INNERCLASS_CONTEXT;
                    if (thisPriority != 0) {
                    }
                    if (otherName.equals("java.lang.Object")) {
                    }
                case 'F':
                    thisPriority = 'H';
                    if (thisPriority != 0) {
                    }
                    if (otherName.equals("java.lang.Object")) {
                    }
                case 'I':
                    thisPriority = Access.CLASS_CONTEXT;
                    if (thisPriority != 0) {
                    }
                    if (otherName.equals("java.lang.Object")) {
                    }
                case 'J':
                    thisPriority = 'D';
                    if (thisPriority != 0) {
                    }
                    if (otherName.equals("java.lang.Object")) {
                    }
                case 'S':
                    thisPriority = 'B';
                    if (thisPriority != 0) {
                    }
                    if (otherName.equals("java.lang.Object")) {
                    }
                case 'V':
                    return 1;
                case 'Z':
                    if (otherName.equals("java.lang.Boolean")) {
                        return 0;
                    }
                    if (otherName.equals("java.lang.Character")) {
                    }
                    if (thisPriority != 0) {
                    }
                    if (otherName.equals("java.lang.Object")) {
                    }
                default:
                    if (thisPriority != 0) {
                    }
                    if (otherName.equals("java.lang.Object")) {
                    }
            }
        }
    }
}
