package gnu.kawa.functions;

import gnu.bytecode.CodeAttr;
import gnu.bytecode.Label;
import gnu.bytecode.Method;
import gnu.bytecode.Type;
import gnu.expr.ApplyExp;
import gnu.expr.Compilation;
import gnu.expr.ConditionalTarget;
import gnu.expr.Expression;
import gnu.expr.IfExp;
import gnu.expr.Inlineable;
import gnu.expr.Language;
import gnu.expr.PrimProcedure;
import gnu.expr.QuoteExp;
import gnu.expr.ReferenceExp;
import gnu.expr.StackTarget;
import gnu.expr.Target;
import gnu.mapping.Procedure;
import gnu.mapping.ProcedureN;
import gnu.math.IntNum;
import gnu.math.Numeric;
import gnu.math.RatNum;
import java.math.BigDecimal;
import java.math.BigInteger;
/* loaded from: classes.dex */
public class NumberCompare extends ProcedureN implements Inlineable {
    static final int RESULT_EQU = 0;
    static final int RESULT_GRT = 1;
    static final int RESULT_LSS = -1;
    static final int RESULT_NAN = -2;
    static final int RESULT_NEQ = -3;
    public static final int TRUE_IF_EQU = 8;
    public static final int TRUE_IF_GRT = 16;
    public static final int TRUE_IF_LSS = 4;
    public static final int TRUE_IF_NAN = 2;
    public static final int TRUE_IF_NEQ = 1;
    int flags;
    Language language;

    @Override // gnu.mapping.Procedure
    public int numArgs() {
        return -4094;
    }

    public static boolean $Eq$V(Object arg1, Object arg2, Object arg3, Object[] rest) {
        if (apply2(8, arg1, arg2) && apply2(8, arg2, arg3)) {
            return rest.length == 0 || (apply2(8, arg3, rest[0]) && applyN(8, rest));
        }
        return false;
    }

    public static boolean $Gr$V(Object arg1, Object arg2, Object arg3, Object[] rest) {
        if (apply2(16, arg1, arg2) && apply2(16, arg2, arg3)) {
            return rest.length == 0 || (apply2(16, arg3, rest[0]) && applyN(16, rest));
        }
        return false;
    }

    public static boolean $Gr$Eq$V(Object arg1, Object arg2, Object arg3, Object[] rest) {
        if (apply2(24, arg1, arg2) && apply2(24, arg2, arg3)) {
            return rest.length == 0 || (apply2(24, arg3, rest[0]) && applyN(24, rest));
        }
        return false;
    }

    public static boolean $Ls$V(Object arg1, Object arg2, Object arg3, Object[] rest) {
        if (apply2(4, arg1, arg2) && apply2(4, arg2, arg3)) {
            return rest.length == 0 || (apply2(4, arg3, rest[0]) && applyN(4, rest));
        }
        return false;
    }

    public static boolean $Ls$Eq$V(Object arg1, Object arg2, Object arg3, Object[] rest) {
        if (apply2(12, arg1, arg2) && apply2(12, arg2, arg3)) {
            return rest.length == 0 || (apply2(12, arg3, rest[0]) && applyN(12, rest));
        }
        return false;
    }

    public static NumberCompare make(Language language, String name, int flags) {
        NumberCompare proc = new NumberCompare();
        proc.language = language;
        proc.setName(name);
        proc.flags = flags;
        proc.setProperty(Procedure.validateApplyKey, "gnu.kawa.functions.CompileArith:validateApplyNumberCompare");
        return proc;
    }

    protected final Language getLanguage() {
        return this.language;
    }

    @Override // gnu.mapping.ProcedureN, gnu.mapping.Procedure
    public Object apply2(Object arg1, Object arg2) {
        return getLanguage().booleanObject(apply2(this.flags, arg1, arg2));
    }

    public static boolean apply2(int flags, Object arg1, Object arg2) {
        return ((1 << (compare(arg1, arg2, true) + 3)) & flags) != 0;
    }

    public static boolean checkCompareCode(int code, int flags) {
        return ((1 << (code + 3)) & flags) != 0;
    }

    public static boolean applyWithPromotion(int flags, Object arg1, Object arg2) {
        return checkCompareCode(compare(arg1, arg2, false), flags);
    }

    public static int compare(Object arg1, Object arg2, boolean exact) {
        int code1 = Arithmetic.classifyValue(arg1);
        int code2 = Arithmetic.classifyValue(arg2);
        return compare(arg1, code1, arg2, code2, exact);
    }

    /* JADX WARN: Removed duplicated region for block: B:54:0x00ca A[RETURN, SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:56:0x00cd  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static int compare(Object arg1, int code1, Object arg2, int code2, boolean exact) {
        double d1;
        double d2;
        if (code1 < 0 || code2 < 0) {
            return -3;
        }
        int code = code1 < code2 ? code2 : code1;
        switch (code) {
            case 1:
                int i1 = Arithmetic.asInt(arg1);
                int i2 = Arithmetic.asInt(arg2);
                if (i1 < i2) {
                    return -1;
                }
                return i1 > i2 ? 1 : 0;
            case 2:
                long l1 = Arithmetic.asLong(arg1);
                long l2 = Arithmetic.asLong(arg2);
                if (l1 < l2) {
                    return -1;
                }
                return l1 > l2 ? 1 : 0;
            case 3:
                BigInteger bi1 = Arithmetic.asBigInteger(arg1);
                BigInteger bi2 = Arithmetic.asBigInteger(arg2);
                return bi1.compareTo(bi2);
            case 4:
                return IntNum.compare(Arithmetic.asIntNum(arg1), Arithmetic.asIntNum(arg2));
            case 5:
                BigDecimal bd1 = Arithmetic.asBigDecimal(arg1);
                BigDecimal bd2 = Arithmetic.asBigDecimal(arg2);
                return bd1.compareTo(bd2);
            case 6:
                return RatNum.compare(Arithmetic.asRatNum(arg1), Arithmetic.asRatNum(arg2));
            case 7:
                if (!exact || (code1 > 6 && code2 > 6)) {
                    float f1 = Arithmetic.asFloat(arg1);
                    float f2 = Arithmetic.asFloat(arg2);
                    if (f1 > f2) {
                        return 1;
                    }
                    if (f1 < f2) {
                        return -1;
                    }
                    return f1 == f2 ? 0 : -2;
                }
                if (exact || (code1 > 6 && code2 > 6)) {
                    d1 = Arithmetic.asDouble(arg1);
                    d2 = Arithmetic.asDouble(arg2);
                    if (d1 <= d2) {
                        return 1;
                    }
                    if (d1 < d2) {
                        return -1;
                    }
                    return d1 == d2 ? 0 : -2;
                }
                Numeric num1 = Arithmetic.asNumeric(arg1);
                Numeric num2 = Arithmetic.asNumeric(arg2);
                return num1.compare(num2);
            case 8:
            case 9:
                if (exact) {
                    break;
                }
                d1 = Arithmetic.asDouble(arg1);
                d2 = Arithmetic.asDouble(arg2);
                if (d1 <= d2) {
                }
                break;
            default:
                Numeric num12 = Arithmetic.asNumeric(arg1);
                Numeric num22 = Arithmetic.asNumeric(arg2);
                return num12.compare(num22);
        }
    }

    static boolean applyN(int flags, Object[] args) {
        for (int i = 0; i < args.length - 1; i++) {
            Object arg1 = args[i];
            Object arg2 = args[i + 1];
            if (!apply2(flags, arg1, arg2)) {
                return false;
            }
        }
        return true;
    }

    @Override // gnu.mapping.ProcedureN, gnu.mapping.Procedure
    public Object applyN(Object[] args) {
        return getLanguage().booleanObject(applyN(this.flags, args));
    }

    @Override // gnu.expr.Inlineable
    public void compile(ApplyExp exp, Compilation comp, Target target) {
        Type commonType;
        int opcode;
        Expression[] args = exp.getArgs();
        if (args.length == 2) {
            Expression arg0 = args[0];
            Expression arg1 = args[1];
            int kind0 = classify(arg0);
            int kind1 = classify(arg1);
            CodeAttr code = comp.getCode();
            if (kind0 > 0 && kind1 > 0 && kind0 <= 10 && kind1 <= 10 && (kind0 != 6 || kind1 != 6)) {
                if (!(target instanceof ConditionalTarget)) {
                    IfExp.compile(exp, QuoteExp.trueExp, QuoteExp.falseExp, comp, target);
                    return;
                }
                int mask = this.flags;
                if (mask == 1) {
                    mask = 20;
                }
                if (kind0 <= 4 && kind1 <= 4 && (kind0 > 2 || kind1 > 2)) {
                    Type[] ctypes = new Type[2];
                    ctypes[0] = Arithmetic.typeIntNum;
                    if (kind1 <= 2) {
                        ctypes[1] = Type.longType;
                    } else if (kind0 <= 2 && ((arg0 instanceof QuoteExp) || (arg1 instanceof QuoteExp) || (arg0 instanceof ReferenceExp) || (arg1 instanceof ReferenceExp))) {
                        ctypes[1] = Type.longType;
                        args = new Expression[]{arg1, arg0};
                        if (mask != 8 && mask != 20) {
                            mask ^= 20;
                        }
                    } else {
                        ctypes[1] = Arithmetic.typeIntNum;
                    }
                    Method cmeth = Arithmetic.typeIntNum.getMethod("compare", ctypes);
                    PrimProcedure compare = new PrimProcedure(cmeth);
                    arg0 = new ApplyExp(compare, args);
                    arg1 = new QuoteExp(IntNum.zero());
                    kind1 = 1;
                    kind0 = 1;
                }
                if (kind0 <= 1 && kind1 <= 1) {
                    commonType = Type.intType;
                } else if (kind0 <= 2 && kind1 <= 2) {
                    commonType = Type.longType;
                } else {
                    commonType = Type.doubleType;
                }
                StackTarget subTarget = new StackTarget(commonType);
                ConditionalTarget ctarget = (ConditionalTarget) target;
                if ((arg0 instanceof QuoteExp) && !(arg1 instanceof QuoteExp)) {
                    Expression tmp = arg1;
                    arg1 = arg0;
                    arg0 = tmp;
                    if (mask != 8 && mask != 20) {
                        mask ^= 20;
                    }
                }
                Label label1 = ctarget.trueBranchComesFirst ? ctarget.ifFalse : ctarget.ifTrue;
                if (ctarget.trueBranchComesFirst) {
                    mask ^= 28;
                }
                switch (mask) {
                    case 4:
                        opcode = 155;
                        break;
                    case 8:
                        opcode = 153;
                        break;
                    case 12:
                        opcode = 158;
                        break;
                    case 16:
                        opcode = 157;
                        break;
                    case 20:
                        opcode = 154;
                        break;
                    case 24:
                        opcode = 156;
                        break;
                    default:
                        opcode = 0;
                        break;
                }
                arg0.compile(comp, subTarget);
                if (kind0 <= 1 && kind1 <= 1 && (arg1 instanceof QuoteExp)) {
                    Object value = ((QuoteExp) arg1).getValue();
                    if ((value instanceof IntNum) && ((IntNum) value).isZero()) {
                        code.emitGotoIfCompare1(label1, opcode);
                        ctarget.emitGotoFirstBranch(code);
                        return;
                    }
                }
                arg1.compile(comp, subTarget);
                code.emitGotoIfCompare2(label1, opcode);
                ctarget.emitGotoFirstBranch(code);
                return;
            }
        }
        ApplyExp.compile(exp, comp, target);
    }

    static int classify(Expression exp) {
        Type type = exp.getType();
        int kind = Arithmetic.classifyType(type);
        if (kind == 4 && (exp instanceof QuoteExp)) {
            Object value = ((QuoteExp) exp).getValue();
            if (value instanceof IntNum) {
                int ilength = ((IntNum) value).intLength();
                if (ilength < 32) {
                    return 1;
                }
                if (ilength < 64) {
                    return 2;
                }
                return kind;
            }
            return kind;
        }
        return kind;
    }

    @Override // gnu.mapping.Procedure
    public Type getReturnType(Expression[] args) {
        return Type.booleanType;
    }
}
