package gnu.kawa.lispexpr;

import gnu.lists.Pair;
import gnu.mapping.Values;
import gnu.text.Lexer;
import gnu.text.LineBufferedReader;
import gnu.text.SyntaxException;
import java.io.IOException;
/* loaded from: classes.dex */
public class ReaderParens extends ReadTableEntry {
    private static ReaderParens instance;
    char close;
    Object command;
    int kind;
    char open;

    @Override // gnu.kawa.lispexpr.ReadTableEntry
    public int getKind() {
        return this.kind;
    }

    public static ReaderParens getInstance(char open, char close) {
        return getInstance(open, close, 5);
    }

    public static ReaderParens getInstance(char open, char close, int kind) {
        if (open == '(' && close == ')' && kind == 5) {
            if (instance == null) {
                instance = new ReaderParens(open, close, kind, null);
            }
            return instance;
        }
        return new ReaderParens(open, close, kind, null);
    }

    public static ReaderParens getInstance(char open, char close, int kind, Object command) {
        return command == null ? getInstance(open, close, kind) : new ReaderParens(open, close, kind, command);
    }

    public ReaderParens(char open, char close, int kind, Object command) {
        this.open = open;
        this.close = close;
        this.kind = kind;
        this.command = command;
    }

    @Override // gnu.kawa.lispexpr.ReadTableEntry
    public Object read(Lexer in, int ch, int count) throws IOException, SyntaxException {
        Object r = readList((LispReader) in, ch, count, this.close);
        if (this.command != null) {
            LineBufferedReader port = in.getPort();
            int startLine = port.getLineNumber();
            int startColumn = port.getColumnNumber();
            Pair p = ((LispReader) in).makePair(this.command, startLine, startColumn);
            ((LispReader) in).setCdr(p, r);
            return p;
        }
        return r;
    }

    /* JADX WARN: Code restructure failed: missing block: B:24:0x0082, code lost:
        r20.error("unexpected '" + ((char) r23) + "' after '.'");
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static Object readList(LispReader lexer, int ch, int count, int close) throws IOException, SyntaxException {
        ReadTableEntry entry;
        Object list;
        Object value;
        LineBufferedReader port = lexer.getPort();
        char saveReadState = lexer.pushNesting(close == 93 ? '[' : '(');
        int startLine = port.getLineNumber();
        int startColumn = port.getColumnNumber();
        Object last = null;
        try {
            Object list2 = lexer.makeNil();
            boolean sawDot = false;
            boolean sawDotCdr = false;
            ReadTable readTable = ReadTable.getCurrent();
            while (true) {
                int line = port.getLineNumber();
                int column = port.getColumnNumber();
                int ch2 = port.read();
                if (ch2 == close) {
                    break;
                }
                if (ch2 < 0) {
                    lexer.eofError("unexpected EOF in list starting here", startLine + 1, startColumn);
                }
                if (ch2 == 46) {
                    ch2 = port.peek();
                    entry = readTable.lookup(ch2);
                    int kind = entry.getKind();
                    if (kind == 1 || kind == 5 || kind == 0) {
                        port.skip();
                        column++;
                        if (ch2 == close) {
                            break;
                        }
                        if (ch2 < 0) {
                            lexer.eofError("unexpected EOF in list starting here", startLine + 1, startColumn);
                        }
                        if (sawDot) {
                            lexer.error("multiple '.' in list");
                            sawDotCdr = false;
                            list2 = lexer.makeNil();
                            last = null;
                        }
                        sawDot = true;
                    } else {
                        ch2 = 46;
                        entry = ReadTableEntry.getConstituentInstance();
                    }
                    list = list2;
                } else {
                    entry = readTable.lookup(ch2);
                    list = list2;
                }
                Object value2 = lexer.readValues(ch2, entry, readTable);
                if (value2 == Values.empty) {
                    list2 = list;
                } else {
                    Object value3 = lexer.handlePostfix(value2, readTable, line, column);
                    if (sawDotCdr) {
                        lexer.error("multiple values after '.'");
                        last = null;
                        list2 = lexer.makeNil();
                        sawDotCdr = false;
                    } else {
                        if (sawDot) {
                            sawDotCdr = true;
                            value = value3;
                        } else {
                            if (last == null) {
                                line = startLine;
                                column = startColumn - 1;
                            }
                            value = lexer.makePair(value3, line, column);
                        }
                        if (last == null) {
                            list = value;
                        } else {
                            lexer.setCdr(last, value);
                        }
                        last = value;
                        list2 = list;
                    }
                }
            }
            return list2;
        } finally {
            lexer.popNesting(saveReadState);
        }
    }
}
