package gnu.kawa.reflect;

import gnu.bytecode.ArrayType;
import gnu.bytecode.ClassType;
import gnu.bytecode.ObjectType;
import gnu.bytecode.Type;
import gnu.expr.ApplyExp;
import gnu.expr.Compilation;
import gnu.expr.Expression;
import gnu.expr.Keyword;
import gnu.expr.Language;
import gnu.expr.PairClassType;
import gnu.expr.PrimProcedure;
import gnu.expr.TypeValue;
import gnu.kawa.lispexpr.ClassNamespace;
import gnu.lists.FString;
import gnu.mapping.CallContext;
import gnu.mapping.MethodProc;
import gnu.mapping.Procedure;
import gnu.mapping.ProcedureN;
import gnu.mapping.Symbol;
import gnu.mapping.WrongType;
import java.lang.reflect.Array;
/* loaded from: classes.dex */
public class Invoke extends ProcedureN {
    char kind;
    Language language;
    public static final Invoke invoke = new Invoke("invoke", '*');
    public static final Invoke invokeStatic = new Invoke("invoke-static", 'S');
    public static final Invoke invokeSpecial = new Invoke("invoke-special", 'P');
    public static final Invoke make = new Invoke("make", 'N');

    public Invoke(String name, char kind) {
        this(name, kind, Language.getDefaultLanguage());
    }

    public Invoke(String name, char kind, Language language) {
        super(name);
        this.kind = kind;
        this.language = language;
        setProperty(Procedure.validateApplyKey, "gnu.kawa.reflect.CompileInvoke:validateApplyInvoke");
    }

    private static ObjectType typeFrom(Object arg, Invoke thisProc) {
        if (arg instanceof Class) {
            arg = Type.make((Class) arg);
        }
        if (arg instanceof ObjectType) {
            return (ObjectType) arg;
        }
        if ((arg instanceof String) || (arg instanceof FString)) {
            return ClassType.make(arg.toString());
        }
        if (arg instanceof Symbol) {
            return ClassType.make(((Symbol) arg).getName());
        }
        if (arg instanceof ClassNamespace) {
            return ((ClassNamespace) arg).getClassType();
        }
        throw new WrongType(thisProc, 0, arg, "class-specifier");
    }

    @Override // gnu.mapping.Procedure
    public void apply(CallContext ctx) throws Throwable {
        Object[] args = ctx.getArgs();
        if (this.kind == 'S' || this.kind == 'V' || this.kind == 's' || this.kind == '*') {
            int nargs = args.length;
            Procedure.checkArgCount(this, nargs);
            Object arg0 = args[0];
            ObjectType dtype = (ObjectType) ((this.kind == 'S' || this.kind == 's') ? typeFrom(arg0, this) : Type.make(arg0.getClass()));
            Procedure proc = lookupMethods(dtype, args[1]);
            Object[] margs = new Object[nargs - (this.kind == 'S' ? 2 : 1)];
            int i = 0;
            if (this.kind == 'V' || this.kind == '*') {
                int i2 = 0 + 1;
                margs[0] = args[0];
                i = i2;
            }
            System.arraycopy(args, 2, margs, i, nargs - 2);
            proc.checkN(margs, ctx);
            return;
        }
        ctx.writeValue(applyN(args));
    }

    /* JADX WARN: Removed duplicated region for block: B:37:0x0122  */
    @Override // gnu.mapping.ProcedureN, gnu.mapping.Procedure
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public Object applyN(Object[] args) throws Throwable {
        Object obj;
        Object result;
        int length;
        int i;
        boolean lengthSpecified;
        Procedure constructor;
        if (this.kind == 'P') {
            throw new RuntimeException(getName() + ": invoke-special not allowed at run time");
        }
        int nargs = args.length;
        Procedure.checkArgCount(this, nargs);
        Object arg0 = args[0];
        ObjectType dtype = (this.kind == 'V' || this.kind == '*') ? (ObjectType) Type.make(arg0.getClass()) : typeFrom(arg0, this);
        if (this.kind == 'N') {
            obj = null;
            if ((dtype instanceof TypeValue) && (constructor = ((TypeValue) dtype).getConstructor()) != null) {
                int nargs2 = nargs - 1;
                Object[] xargs = new Object[nargs2];
                System.arraycopy(args, 1, xargs, 0, nargs2);
                return constructor.applyN(xargs);
            }
            if (dtype instanceof PairClassType) {
                PairClassType ptype = (PairClassType) dtype;
                dtype = ptype.instanceType;
            }
            if (dtype instanceof ArrayType) {
                Type elementType = ((ArrayType) dtype).getComponentType();
                int len = args.length - 1;
                if (len >= 2 && (args[1] instanceof Keyword)) {
                    String name = ((Keyword) args[1]).getName();
                    if ("length".equals(name) || "size".equals(name)) {
                        length = ((Number) args[2]).intValue();
                        i = 3;
                        lengthSpecified = true;
                        Object arr = Array.newInstance(elementType.getReflectClass(), length);
                        int index = 0;
                        while (i <= len) {
                            Object arg = args[i];
                            if (lengthSpecified && (arg instanceof Keyword) && i < len) {
                                String kname = ((Keyword) arg).getName();
                                try {
                                    index = Integer.parseInt(kname);
                                    i++;
                                    arg = args[i];
                                } catch (Throwable th) {
                                    throw new RuntimeException("non-integer keyword '" + kname + "' in array constructor");
                                }
                            }
                            Array.set(arr, index, elementType.coerceFromObject(arg));
                            index++;
                            i++;
                        }
                        return arr;
                    }
                }
                length = len;
                i = 1;
                lengthSpecified = false;
                Object arr2 = Array.newInstance(elementType.getReflectClass(), length);
                int index2 = 0;
                while (i <= len) {
                }
                return arr2;
            }
        } else {
            obj = args[1];
        }
        MethodProc proc = lookupMethods(dtype, obj);
        if (this.kind != 'N') {
            Object[] margs = new Object[nargs - ((this.kind == 'S' || this.kind == 's') ? 2 : 1)];
            int i2 = 0;
            if (this.kind == 'V' || this.kind == '*') {
                int i3 = 0 + 1;
                margs[0] = args[0];
                i2 = i3;
            }
            System.arraycopy(args, 2, margs, i2, nargs - 2);
            return proc.applyN(margs);
        }
        CallContext vars = CallContext.getInstance();
        int keywordStart = 0;
        while (keywordStart < args.length && !(args[keywordStart] instanceof Keyword)) {
            keywordStart++;
        }
        int err = -1;
        if (keywordStart == args.length) {
            err = proc.matchN(args, vars);
            if (err == 0) {
                return vars.runUntilValue();
            }
            MethodProc vproc = ClassMethods.apply((ClassType) dtype, "valueOf", (char) 0, this.language);
            if (vproc != null) {
                Object[] margs2 = new Object[nargs - 1];
                System.arraycopy(args, 1, margs2, 0, nargs - 1);
                err = vproc.matchN(margs2, vars);
                if (err == 0) {
                    return vars.runUntilValue();
                }
            }
            result = proc.apply1(args[0]);
        } else {
            Object[] cargs = new Object[keywordStart];
            System.arraycopy(args, 0, cargs, 0, keywordStart);
            result = proc.applyN(cargs);
        }
        int i4 = keywordStart;
        while (i4 + 1 < args.length) {
            Object arg2 = args[i4];
            if (!(arg2 instanceof Keyword)) {
                break;
            }
            Keyword key = (Keyword) arg2;
            SlotSet.apply(false, result, key.getName(), args[i4 + 1]);
            i4 += 2;
        }
        if (keywordStart == args.length) {
            i4 = 1;
        }
        if (i4 != args.length) {
            MethodProc aproc = ClassMethods.apply((ClassType) dtype, "add", (char) 0, this.language);
            if (aproc == null) {
                throw MethodProc.matchFailAsException(err, proc, args);
            }
            while (i4 < args.length) {
                aproc.apply2(result, args[i4]);
                i4++;
            }
        }
        return result;
    }

    @Override // gnu.mapping.Procedure
    public int numArgs() {
        return (this.kind == 'N' ? 1 : 2) | (-4096);
    }

    protected MethodProc lookupMethods(ObjectType dtype, Object name) {
        String mname;
        String mname2;
        char c = 'P';
        if (this.kind == 'N') {
            mname2 = "<init>";
        } else {
            if ((name instanceof String) || (name instanceof FString)) {
                mname = name.toString();
            } else if (name instanceof Symbol) {
                mname = ((Symbol) name).getName();
            } else {
                throw new WrongType((Procedure) this, 1, (ClassCastException) null);
            }
            mname2 = Compilation.mangleName(mname);
        }
        if (this.kind != 'P') {
            c = (this.kind == '*' || this.kind == 'V') ? 'V' : (char) 0;
        }
        MethodProc proc = ClassMethods.apply(dtype, mname2, c, this.language);
        if (proc == null) {
            throw new RuntimeException(getName() + ": no method named `" + mname2 + "' in class " + dtype.getName());
        }
        return proc;
    }

    public static synchronized ApplyExp makeInvokeStatic(ClassType type, String name, Expression[] args) {
        ApplyExp applyExp;
        synchronized (Invoke.class) {
            PrimProcedure method = getStaticMethod(type, name, args);
            if (method == null) {
                throw new RuntimeException("missing or ambiguous method `" + name + "' in " + type.getName());
            }
            applyExp = new ApplyExp(method, args);
        }
        return applyExp;
    }

    public static synchronized PrimProcedure getStaticMethod(ClassType type, String name, Expression[] args) {
        PrimProcedure staticMethod;
        synchronized (Invoke.class) {
            staticMethod = CompileInvoke.getStaticMethod(type, name, args);
        }
        return staticMethod;
    }
}
