package gnu.kawa.util;

import com.google.appinventor.components.runtime.util.Ev3Constants;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Hashtable;
import java.util.StringTokenizer;
/* loaded from: classes.dex */
public class PreProcess {
    static final String JAVA4_FEATURES = "+JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio";
    static final String JAVA5_FEATURES = "+JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio +use:org.w3c.dom.Node +use:javax.xml.transform +JAXP-1.3 -JAXP-QName";
    static final String NO_JAVA4_FEATURES = "-JAVA5 -use:java.util.IdentityHashMap -use:java.lang.CharSequence -use:java.lang.Throwable.getCause -use:java.net.URI -use:java.util.regex -use:org.w3c.dom.Node -JAXP-1.3 -use:javax.xml.transform -JAVA5 -JAVA6 -JAVA6COMPAT5 -JAXP-QName -use:java.text.Normalizer -SAX2 -use:java.nio -Android";
    static final String NO_JAVA6_FEATURES = "-JAVA6 -JAVA7 -use:java.dyn -use:java.text.Normalizer";
    static String[] version_features = {"java1", "-JAVA2 -JAVA5 -use:java.util.IdentityHashMap -use:java.lang.CharSequence -use:java.lang.Throwable.getCause -use:java.net.URI -use:java.util.regex -use:org.w3c.dom.Node -JAXP-1.3 -use:javax.xml.transform -JAVA5 -JAVA6 -JAVA6COMPAT5 -JAXP-QName -use:java.text.Normalizer -SAX2 -use:java.nio -Android -JAVA6 -JAVA7 -use:java.dyn -use:java.text.Normalizer", "java2", "+JAVA2 -JAVA5 -use:java.util.IdentityHashMap -use:java.lang.CharSequence -use:java.lang.Throwable.getCause -use:java.net.URI -use:java.util.regex -use:org.w3c.dom.Node -JAXP-1.3 -use:javax.xml.transform -JAVA5 -JAVA6 -JAVA6COMPAT5 -JAXP-QName -use:java.text.Normalizer -SAX2 -use:java.nio -Android -JAVA6 -JAVA7 -use:java.dyn -use:java.text.Normalizer", "java4", "-JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio -use:org.w3c.dom.Node -JAXP-1.3 -use:javax.xml.transform -JAXP-QName -JAVA6COMPAT5 -Android -JAVA6 -JAVA7 -use:java.dyn -use:java.text.Normalizer", "java4x", "-JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio +use:org.w3c.dom.Node +JAXP-1.3 +use:javax.xml.transform -JAXP-QName -JAVA6COMPAT5 -Android -JAVA6 -JAVA7 -use:java.dyn -use:java.text.Normalizer", "java5", "+JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio +use:org.w3c.dom.Node +use:javax.xml.transform +JAXP-1.3 -JAXP-QName -JAVA6COMPAT5 -Android -JAVA6 -JAVA7 -use:java.dyn -use:java.text.Normalizer", "java6compat5", "+JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio +use:org.w3c.dom.Node +use:javax.xml.transform +JAXP-1.3 -JAXP-QName -JAVA6 -JAVA7 +JAVA6COMPAT5 +use:java.text.Normalizer -use:java.dyn -Android", "java6", "+JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio +use:org.w3c.dom.Node +use:javax.xml.transform +JAXP-1.3 -JAXP-QName +JAVA6 -JAVA7 -JAVA6COMPAT5 +use:java.text.Normalizer -use:java.dyn -Android", "java7", "+JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio +use:org.w3c.dom.Node +use:javax.xml.transform +JAXP-1.3 -JAXP-QName +JAVA6 +JAVA7 -JAVA6COMPAT5 +use:java.text.Normalizer +use:java.dyn -Android", "android", "+JAVA5 +JAVA2 +use:java.util.IdentityHashMap +use:java.lang.CharSequence +use:java.lang.Throwable.getCause +use:java.net.URI +use:java.util.regex +SAX2 +use:java.nio +use:org.w3c.dom.Node +JAXP-1.3 -JAXP-QName -use:javax.xml.transform -JAVA6 -JAVA6COMPAT5 +Android -JAVA6 -JAVA7 -use:java.dyn -use:java.text.Normalizer"};
    String filename;
    Hashtable keywords = new Hashtable();
    int lineno;
    byte[] resultBuffer;
    int resultLength;

    void error(String msg) {
        System.err.println(this.filename + ':' + this.lineno + ": " + msg);
        System.exit(-1);
    }

    public void filter(String filename) throws Throwable {
        if (filter(filename, new BufferedInputStream(new FileInputStream(filename)))) {
            FileOutputStream out = new FileOutputStream(filename);
            out.write(this.resultBuffer, 0, this.resultLength);
            out.close();
            System.err.println("Pre-processed " + filename);
        }
    }

    /* JADX WARN: Code restructure failed: missing block: B:19:0x009a, code lost:
        if (r15 >= 0) goto L194;
     */
    /* JADX WARN: Code restructure failed: missing block: B:20:0x009c, code lost:
        if (r8 > 0) goto L194;
     */
    /* JADX WARN: Code restructure failed: missing block: B:22:0x00a2, code lost:
        if (r6 == 13) goto L194;
     */
    /* JADX WARN: Code restructure failed: missing block: B:24:0x00a8, code lost:
        if (r6 == 10) goto L194;
     */
    /* JADX WARN: Code restructure failed: missing block: B:25:0x00aa, code lost:
        if (r12 == r13) goto L25;
     */
    /* JADX WARN: Code restructure failed: missing block: B:27:0x00b0, code lost:
        if (r6 == 32) goto L194;
     */
    /* JADX WARN: Code restructure failed: missing block: B:29:0x00b6, code lost:
        if (r6 == 9) goto L194;
     */
    /* JADX WARN: Code restructure failed: missing block: B:31:0x00bc, code lost:
        if (r6 != 47) goto L193;
     */
    /* JADX WARN: Code restructure failed: missing block: B:32:0x00be, code lost:
        r33.mark(100);
        r14 = r33.read();
     */
    /* JADX WARN: Code restructure failed: missing block: B:33:0x00cf, code lost:
        if (r14 != 47) goto L177;
     */
    /* JADX WARN: Code restructure failed: missing block: B:34:0x00d1, code lost:
        r16 = false;
     */
    /* JADX WARN: Code restructure failed: missing block: B:35:0x00d3, code lost:
        r33.reset();
     */
    /* JADX WARN: Code restructure failed: missing block: B:36:0x00d6, code lost:
        if (r16 == false) goto L194;
     */
    /* JADX WARN: Code restructure failed: missing block: B:37:0x00d8, code lost:
        r21 = r20 + 1;
        r5[r20] = com.google.appinventor.components.runtime.util.Ev3Constants.Opcode.INIT_BYTES;
        r20 = r21 + 1;
        r5[r21] = com.google.appinventor.components.runtime.util.Ev3Constants.Opcode.INIT_BYTES;
        r21 = r20 + 1;
        r5[r20] = 32;
        r8 = 1;
        r7 = true;
     */
    /* JADX WARN: Code restructure failed: missing block: B:52:0x0114, code lost:
        if (r14 != 42) goto L192;
     */
    /* JADX WARN: Code restructure failed: missing block: B:53:0x0116, code lost:
        r14 = r33.read();
     */
    /* JADX WARN: Code restructure failed: missing block: B:54:0x011e, code lost:
        if (r14 == 32) goto L191;
     */
    /* JADX WARN: Code restructure failed: missing block: B:56:0x0124, code lost:
        if (r14 == 9) goto L189;
     */
    /* JADX WARN: Code restructure failed: missing block: B:58:0x012a, code lost:
        if (r14 == 35) goto L188;
     */
    /* JADX WARN: Code restructure failed: missing block: B:59:0x012c, code lost:
        r16 = true;
     */
    /* JADX WARN: Code restructure failed: missing block: B:61:0x012f, code lost:
        r16 = false;
     */
    /* JADX WARN: Code restructure failed: missing block: B:62:0x0132, code lost:
        r16 = true;
     */
    /* JADX WARN: Code restructure failed: missing block: B:63:0x0135, code lost:
        r16 = true;
     */
    /* JADX WARN: Removed duplicated region for block: B:116:0x0212  */
    /* JADX WARN: Removed duplicated region for block: B:74:0x0165  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public boolean filter(String filename, BufferedInputStream in) throws Throwable {
        int len;
        int firstNonSpace;
        int lastNonSpace;
        int i;
        int firstNonSpace2;
        int lastNonSpace2;
        String rest;
        Object obj;
        this.filename = filename;
        boolean changed = false;
        byte[] buf = new byte[2000];
        int len2 = 0;
        int lineStart = 0;
        int dataStart = -1;
        int cmdLine = 0;
        this.lineno = 1;
        int commentAt = -1;
        int curIndent = 0;
        int nesting = 0;
        int skipNesting = 0;
        String cmd = null;
        int changedLine = 0;
        while (true) {
            int c = in.read();
            if (c < 0) {
                break;
            }
            if (len2 + 10 >= buf.length) {
                byte[] nbuf = new byte[len2 * 2];
                System.arraycopy(buf, 0, nbuf, 0, len2);
                buf = nbuf;
            }
            if (c == 10 && len2 > 0 && buf[len2 - 1] == 13) {
                buf[len2] = (byte) c;
                len2++;
            } else {
                int len3 = len2;
                if (c != 32 && c != 9 && dataStart < 0) {
                    dataStart = len3;
                    if (nesting > 0 && commentAt != curIndent && c == 47) {
                        c = in.read();
                        if (c < 0) {
                            len2 = len3;
                            break;
                        }
                        if (c != 47) {
                            len = len3 + 1;
                            buf[len3] = Ev3Constants.Opcode.INIT_BYTES;
                        } else {
                            c = in.read();
                            if (c < 0) {
                                len2 = len3;
                                break;
                            }
                            changedLine = -1;
                            changed = true;
                            if (c == 32 && ((c = in.read()) == 32 || c == 9)) {
                                dataStart = -1;
                                len = len3;
                            }
                        }
                        buf[len] = (byte) c;
                        len2 = len + 1;
                        if (c != 13 || c == 10) {
                            firstNonSpace = -1;
                            lastNonSpace = 0;
                            for (i = lineStart; i < len2 - 1; i++) {
                                if (buf[i] != 32 && buf[i] != 9) {
                                    lastNonSpace = i;
                                    if (firstNonSpace < 0) {
                                        firstNonSpace = i;
                                    }
                                }
                            }
                            if (lastNonSpace - firstNonSpace >= 4 && buf[firstNonSpace] == 47 && buf[firstNonSpace + 1] == 42 && buf[lastNonSpace - 1] == 42 && buf[lastNonSpace] == 47) {
                                firstNonSpace2 = firstNonSpace + 2;
                                while (firstNonSpace2 < lastNonSpace && buf[firstNonSpace2] == 32) {
                                    firstNonSpace2++;
                                }
                                lastNonSpace2 = lastNonSpace - 2;
                                while (lastNonSpace2 > firstNonSpace2 && buf[lastNonSpace2] == 32) {
                                    lastNonSpace2--;
                                }
                                if (buf[firstNonSpace2] == 35) {
                                    String cmnt = new String(buf, firstNonSpace2, (lastNonSpace2 - firstNonSpace2) + 1, "ISO-8859-1");
                                    int sp = cmnt.indexOf(32);
                                    cmdLine = this.lineno;
                                    if (sp > 0) {
                                        cmd = cmnt.substring(0, sp);
                                        rest = cmnt.substring(sp).trim();
                                        obj = this.keywords.get(rest);
                                    } else {
                                        cmd = cmnt;
                                        rest = "";
                                        obj = null;
                                    }
                                    if ("#ifdef".equals(cmd) || "#ifndef".equals(cmd)) {
                                        if (obj == null) {
                                            System.err.println(filename + ":" + this.lineno + ": warning - undefined keyword: " + rest);
                                            obj = Boolean.FALSE;
                                        }
                                        nesting++;
                                        if (skipNesting > 0) {
                                            commentAt = curIndent;
                                        } else if ((cmd.charAt(3) == 'n') != (obj == Boolean.FALSE)) {
                                            commentAt = curIndent;
                                            skipNesting = nesting;
                                        }
                                    } else if ("#else".equals(cmd)) {
                                        if (nesting == 0) {
                                            error("unexpected " + cmd);
                                        } else if (nesting == skipNesting) {
                                            commentAt = -1;
                                            skipNesting = 0;
                                        } else {
                                            commentAt = curIndent;
                                            if (skipNesting == 0) {
                                                skipNesting = nesting;
                                            }
                                        }
                                    } else if ("#endif".equals(cmd)) {
                                        if (nesting == 0) {
                                            error("unexpected " + cmd);
                                        }
                                        if (nesting == skipNesting) {
                                            skipNesting = 0;
                                            commentAt = -1;
                                        } else if (skipNesting > 0) {
                                            commentAt = curIndent;
                                        }
                                        nesting--;
                                    } else {
                                        error("unknown command: " + cmnt);
                                    }
                                }
                            }
                            lineStart = len2;
                            dataStart = -1;
                            curIndent = 0;
                            this.lineno++;
                            changedLine = 0;
                        } else if (dataStart < 0) {
                            curIndent = c == 9 ? (curIndent + 8) & (-8) : curIndent + 1;
                        }
                    }
                }
                len = len3;
                buf[len] = (byte) c;
                len2 = len + 1;
                if (c != 13) {
                }
                firstNonSpace = -1;
                lastNonSpace = 0;
                while (i < len2 - 1) {
                }
                if (lastNonSpace - firstNonSpace >= 4) {
                    firstNonSpace2 = firstNonSpace + 2;
                    while (firstNonSpace2 < lastNonSpace) {
                        firstNonSpace2++;
                    }
                    lastNonSpace2 = lastNonSpace - 2;
                    while (lastNonSpace2 > firstNonSpace2) {
                        lastNonSpace2--;
                    }
                    if (buf[firstNonSpace2] == 35) {
                    }
                }
                lineStart = len2;
                dataStart = -1;
                curIndent = 0;
                this.lineno++;
                changedLine = 0;
            }
        }
        if (nesting != 0) {
            this.lineno = cmdLine;
            error("unterminated " + cmd);
        }
        this.resultBuffer = buf;
        this.resultLength = len2;
        return changed;
    }

    void handleArg(String arg) {
        if (arg.charAt(0) == '%') {
            String arg2 = arg.substring(1);
            int i = 0;
            while (true) {
                if (i >= version_features.length) {
                    System.err.println("Unknown version: " + arg2);
                    System.exit(-1);
                }
                if (arg2.equals(version_features[i])) {
                    break;
                }
                i += 2;
            }
            String features = version_features[i + 1];
            System.err.println("(variant " + arg2 + " maps to: " + features + ")");
            StringTokenizer tokenizer = new StringTokenizer(features);
            while (tokenizer.hasMoreTokens()) {
                handleArg(tokenizer.nextToken());
            }
        } else if (arg.charAt(0) == '+') {
            this.keywords.put(arg.substring(1), Boolean.TRUE);
        } else if (arg.charAt(0) == '-') {
            int eq = arg.indexOf(61);
            if (eq > 1) {
                String keyword = arg.substring(arg.charAt(1) == '-' ? 2 : 1, eq);
                String value = arg.substring(eq + 1);
                Boolean b = Boolean.FALSE;
                if (value.equalsIgnoreCase("true")) {
                    b = Boolean.TRUE;
                } else if (!value.equalsIgnoreCase("false")) {
                    System.err.println("invalid value " + value + " for " + keyword);
                    System.exit(-1);
                }
                this.keywords.put(keyword, b);
                return;
            }
            this.keywords.put(arg.substring(1), Boolean.FALSE);
        } else {
            try {
                filter(arg);
            } catch (Throwable ex) {
                System.err.println("caught " + ex);
                ex.printStackTrace();
                System.exit(-1);
            }
        }
    }

    public static void main(String[] args) {
        PreProcess pp = new PreProcess();
        pp.keywords.put("true", Boolean.TRUE);
        pp.keywords.put("false", Boolean.FALSE);
        for (String str : args) {
            pp.handleArg(str);
        }
    }
}
