package gnu.text;

import gnu.lists.CharSeq;
import java.io.Reader;
/* loaded from: classes.dex */
public class QueueReader extends Reader implements Appendable {
    boolean EOFseen;
    char[] buffer;
    int limit;
    int mark;
    int pos;
    int readAheadLimit;

    @Override // java.io.Reader
    public boolean markSupported() {
        return true;
    }

    @Override // java.io.Reader
    public synchronized void mark(int readAheadLimit) {
        this.readAheadLimit = readAheadLimit;
        this.mark = this.pos;
    }

    @Override // java.io.Reader
    public synchronized void reset() {
        if (this.readAheadLimit > 0) {
            this.pos = this.mark;
        }
    }

    void resize(int len) {
        int cur_size = this.limit - this.pos;
        if (this.readAheadLimit > 0 && this.pos - this.mark <= this.readAheadLimit) {
            cur_size = this.limit - this.mark;
        } else {
            this.mark = this.pos;
        }
        char[] new_buffer = this.buffer.length < cur_size + len ? new char[(cur_size * 2) + len] : this.buffer;
        System.arraycopy(this.buffer, this.mark, new_buffer, 0, cur_size);
        this.buffer = new_buffer;
        this.pos -= this.mark;
        this.mark = 0;
        this.limit = cur_size;
    }

    @Override // java.lang.Appendable
    public QueueReader append(CharSequence csq) {
        if (csq == null) {
            csq = "null";
        }
        return append(csq, 0, csq.length());
    }

    @Override // java.lang.Appendable
    public synchronized QueueReader append(CharSequence csq, int start, int end) {
        if (csq == null) {
            csq = "null";
        }
        int len = end - start;
        reserveSpace(len);
        int sz = this.limit;
        char[] d = this.buffer;
        if (csq instanceof String) {
            ((String) csq).getChars(start, end, d, sz);
        } else if (csq instanceof CharSeq) {
            ((CharSeq) csq).getChars(start, end, d, sz);
        } else {
            int i = start;
            int j = sz;
            while (i < end) {
                int j2 = j + 1;
                d[j] = csq.charAt(i);
                i++;
                j = j2;
            }
        }
        this.limit = sz + len;
        notifyAll();
        return this;
    }

    public void append(char[] chars) {
        append(chars, 0, chars.length);
    }

    public synchronized void append(char[] chars, int off, int len) {
        reserveSpace(len);
        System.arraycopy(chars, off, this.buffer, this.limit, len);
        this.limit += len;
        notifyAll();
    }

    @Override // java.lang.Appendable
    public synchronized QueueReader append(char ch) {
        reserveSpace(1);
        char[] cArr = this.buffer;
        int i = this.limit;
        this.limit = i + 1;
        cArr[i] = ch;
        notifyAll();
        return this;
    }

    public synchronized void appendEOF() {
        this.EOFseen = true;
    }

    protected void reserveSpace(int len) {
        if (this.buffer == null) {
            this.buffer = new char[len + 100];
        } else if (this.buffer.length < this.limit + len) {
            resize(len);
        }
    }

    @Override // java.io.Reader
    public synchronized boolean ready() {
        boolean z;
        if (this.pos >= this.limit) {
            z = this.EOFseen;
        }
        return z;
    }

    public void checkAvailable() {
    }

    /* JADX WARN: Code restructure failed: missing block: B:15:0x0017, code lost:
        r1 = r4.buffer;
        r2 = r4.pos;
        r4.pos = r2 + 1;
        r0 = r1[r2];
     */
    @Override // java.io.Reader
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public synchronized int read() {
        char c;
        while (true) {
            if (this.pos < this.limit) {
                break;
            } else if (this.EOFseen) {
                c = 65535;
                break;
            } else {
                checkAvailable();
                try {
                    wait();
                } catch (InterruptedException e) {
                }
            }
        }
        return c;
    }

    /* JADX WARN: Code restructure failed: missing block: B:14:0x0018, code lost:
        r0 = r3.limit - r3.pos;
     */
    /* JADX WARN: Code restructure failed: missing block: B:15:0x001e, code lost:
        if (r6 <= r0) goto L26;
     */
    /* JADX WARN: Code restructure failed: missing block: B:16:0x0020, code lost:
        r6 = r0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:17:0x0021, code lost:
        java.lang.System.arraycopy(r3.buffer, r3.pos, r4, r5, r6);
        r3.pos += r6;
     */
    /* JADX WARN: Code restructure failed: missing block: B:18:0x002d, code lost:
        r1 = r6;
     */
    @Override // java.io.Reader
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public synchronized int read(char[] cbuf, int off, int len) {
        int i;
        if (len != 0) {
            while (true) {
                if (this.pos < this.limit) {
                    break;
                } else if (this.EOFseen) {
                    i = -1;
                    break;
                } else {
                    checkAvailable();
                    try {
                        wait();
                    } catch (InterruptedException e) {
                    }
                }
            }
        } else {
            i = 0;
        }
        return i;
    }

    @Override // java.io.Reader, java.io.Closeable, java.lang.AutoCloseable
    public synchronized void close() {
        this.pos = 0;
        this.limit = 0;
        this.mark = 0;
        this.EOFseen = true;
        this.buffer = null;
    }
}
