package gnu.xml;

import gnu.lists.Consumer;
import gnu.text.LineBufferedReader;
import gnu.text.LineInputStreamReader;
import gnu.text.Path;
import gnu.text.SourceMessages;
import java.io.IOException;
import java.io.InputStream;
/* loaded from: classes.dex */
public class XMLParser {
    private static final int ATTRIBUTE_SEEN_EQ_STATE = 11;
    private static final int ATTRIBUTE_SEEN_NAME_STATE = 8;
    static final String BAD_ENCODING_SYNTAX = "bad 'encoding' declaration";
    static final String BAD_STANDALONE_SYNTAX = "bad 'standalone' declaration";
    private static final int BEGIN_ELEMENT_STATE = 2;
    private static final int DOCTYPE_NAME_SEEN_STATE = 16;
    private static final int DOCTYPE_SEEN_STATE = 13;
    private static final int END_ELEMENT_STATE = 4;
    private static final int EXPECT_NAME_MODIFIER = 1;
    private static final int EXPECT_RIGHT_STATE = 27;
    private static final int INIT_LEFT_QUEST_STATE = 30;
    private static final int INIT_LEFT_STATE = 34;
    private static final int INIT_STATE = 0;
    private static final int INIT_TEXT_STATE = 31;
    private static final int INVALID_VERSION_DECL = 35;
    private static final int MAYBE_ATTRIBUTE_STATE = 10;
    private static final int PREV_WAS_CR_STATE = 28;
    private static final int SAW_AMP_SHARP_STATE = 26;
    private static final int SAW_AMP_STATE = 25;
    private static final int SAW_ENTITY_REF = 6;
    private static final int SAW_EOF_ERROR = 37;
    private static final int SAW_ERROR = 36;
    private static final int SAW_LEFT_EXCL_MINUS_STATE = 22;
    private static final int SAW_LEFT_EXCL_STATE = 20;
    private static final int SAW_LEFT_QUEST_STATE = 21;
    private static final int SAW_LEFT_SLASH_STATE = 19;
    private static final int SAW_LEFT_STATE = 14;
    private static final int SKIP_SPACES_MODIFIER = 2;
    private static final int TEXT_STATE = 1;

    public static void parse(Object uri, SourceMessages messages, Consumer out) throws IOException {
        parse(Path.openInputStream(uri), uri, messages, out);
    }

    public static LineInputStreamReader XMLStreamReader(InputStream strm) throws IOException {
        LineInputStreamReader in = new LineInputStreamReader(strm);
        int b1 = in.getByte();
        int b2 = b1 < 0 ? -1 : in.getByte();
        int b3 = b2 < 0 ? -1 : in.getByte();
        if (b1 == 239 && b2 == 187 && b3 == 191) {
            in.resetStart(3);
            in.setCharset("UTF-8");
        } else if (b1 == 255 && b2 == 254 && b3 != 0) {
            in.resetStart(2);
            in.setCharset("UTF-16LE");
        } else if (b1 == 254 && b2 == 255 && b3 != 0) {
            in.resetStart(2);
            in.setCharset("UTF-16BE");
        } else {
            int b4 = b3 >= 0 ? in.getByte() : -1;
            if (b1 == 76 && b2 == 111 && b3 == 167 && b4 == 148) {
                throw new RuntimeException("XMLParser: EBCDIC encodings not supported");
            }
            in.resetStart(0);
            if ((b1 == 60 && ((b2 == 63 && b3 == 120 && b4 == 109) || (b2 == 0 && b3 == 63 && b4 == 0))) || (b1 == 0 && b2 == 60 && b3 == 0 && b4 == 63)) {
                char[] buffer = in.buffer;
                if (buffer == null) {
                    buffer = new char[8192];
                    in.buffer = buffer;
                }
                int pos = 0;
                int quote = 0;
                while (true) {
                    int b = in.getByte();
                    if (b != 0) {
                        if (b < 0) {
                            break;
                        }
                        int pos2 = pos + 1;
                        buffer[pos] = (char) (b & 255);
                        if (quote == 0) {
                            if (b == 62) {
                                pos = pos2;
                                break;
                            } else if (b == 39 || b == 34) {
                                quote = b;
                            }
                        } else if (b == quote) {
                            quote = 0;
                        }
                        pos = pos2;
                    }
                }
                in.pos = 0;
                in.limit = pos;
            } else {
                in.setCharset("UTF-8");
            }
        }
        in.setKeepFullLines(false);
        return in;
    }

    public static void parse(InputStream strm, Object uri, SourceMessages messages, Consumer out) throws IOException {
        LineInputStreamReader in = XMLStreamReader(strm);
        if (uri != null) {
            in.setName(uri);
        }
        parse((LineBufferedReader) in, messages, out);
        in.close();
    }

    public static void parse(LineBufferedReader in, SourceMessages messages, Consumer out) throws IOException {
        XMLFilter filter = new XMLFilter(out);
        filter.setMessages(messages);
        filter.setSourceLocator(in);
        filter.startDocument();
        Path uri = in.getPath();
        if (uri != null) {
            filter.writeDocumentUri(uri);
        }
        parse(in, filter);
        filter.endDocument();
    }

    public static void parse(LineBufferedReader in, SourceMessages messages, XMLFilter filter) throws IOException {
        filter.setMessages(messages);
        filter.setSourceLocator(in);
        filter.startDocument();
        Path uri = in.getPath();
        if (uri != null) {
            filter.writeDocumentUri(uri);
        }
        parse(in, filter);
        filter.endDocument();
        in.close();
    }

    /* JADX WARN: Code restructure failed: missing block: B:163:0x023d, code lost:
        if (r5 != 0) goto L247;
     */
    /* JADX WARN: Code restructure failed: missing block: B:165:0x0243, code lost:
        if (r22 != 8) goto L256;
     */
    /* JADX WARN: Code restructure failed: missing block: B:166:0x0245, code lost:
        r17 = "missing or invalid attribute name";
     */
    /* JADX WARN: Code restructure failed: missing block: B:167:0x0247, code lost:
        r22 = 36;
        r18 = r19;
     */
    /* JADX WARN: Code restructure failed: missing block: B:169:0x0250, code lost:
        if (r22 == 2) goto L261;
     */
    /* JADX WARN: Code restructure failed: missing block: B:171:0x0255, code lost:
        if (r22 != 4) goto L260;
     */
    /* JADX WARN: Code restructure failed: missing block: B:172:0x0257, code lost:
        r17 = "missing or invalid element name";
     */
    /* JADX WARN: Code restructure failed: missing block: B:173:0x025a, code lost:
        r17 = "missing or invalid name";
     */
    /* JADX WARN: Code restructure failed: missing block: B:186:0x028b, code lost:
        r26.pos = r19;
        r27.error('e', "invalid character reference");
        r22 = 1;
     */
    /* JADX WARN: Code restructure failed: missing block: B:409:0x05c2, code lost:
        r17 = "junk at end of xml declaration";
        r18 = r6;
        r22 = 36;
     */
    /* JADX WARN: Code restructure failed: missing block: B:572:0x0878, code lost:
        r18 = r19;
     */
    /* JADX WARN: Code restructure failed: missing block: B:810:?, code lost:
        return;
     */
    /* JADX WARN: Removed duplicated region for block: B:23:0x006d  */
    /* JADX WARN: Removed duplicated region for block: B:418:0x05f4 A[LOOP:18: B:419:0x05fa->B:418:0x05f4, LOOP_END] */
    /* JADX WARN: Removed duplicated region for block: B:583:0x0806 A[SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:586:0x006a A[SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:622:0x0029 A[SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:803:0x0023 A[SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static void parse(LineBufferedReader in, XMLFilter out) {
        int pos;
        int pos2;
        int pos3;
        int pos4;
        char[] buffer = in.buffer;
        int pos5 = in.pos;
        int limit = in.limit;
        int state = 0;
        char terminator = '<';
        int continue_state = 14;
        char ch = ' ';
        int length = 0;
        int dstart = -1;
        String message = null;
        int start = limit;
        while (true) {
            switch (state) {
                case 0:
                    state = 31;
                    pos = pos5;
                    if (pos < limit) {
                        pos5 = pos + 1;
                        ch = buffer[pos];
                    } else {
                        int saved = pos - start;
                        if (saved > 0) {
                            try {
                                in.pos = start;
                                in.mark(saved + 1);
                            } catch (IOException ex) {
                                throw new RuntimeException(ex.getMessage());
                            }
                        }
                        in.pos = pos;
                        int x = in.read();
                        if (x < 0) {
                            if (state != 1 && state != 28) {
                                state = 37;
                                pos5 = pos;
                            }
                        } else {
                            if (saved > 0) {
                                in.reset();
                                in.skip(saved);
                            } else {
                                in.unread_quick();
                            }
                            int pos6 = in.pos;
                            buffer = in.buffer;
                            limit = in.limit;
                            start = saved > 0 ? pos6 - saved : limit;
                            ch = buffer[pos6];
                            pos5 = pos6 + 1;
                        }
                    }
                    break;
                case 1:
                    start = pos5 - 1;
                    int length2 = pos5;
                    while (true) {
                        pos = pos5;
                        if (ch == terminator) {
                            state = continue_state;
                            pos2 = pos;
                        } else if (ch == '&') {
                            state = 25;
                            pos2 = pos;
                        } else {
                            if (ch == '\r') {
                                length = pos - length2;
                                in.pos = pos;
                                if (length > 0) {
                                    out.textFromParser(buffer, start, length);
                                }
                                if (pos < limit) {
                                    ch = buffer[pos];
                                    if (ch == '\n') {
                                        start = pos;
                                        pos3 = pos + 1;
                                        length2 = pos3;
                                    } else {
                                        out.linefeedFromParser();
                                        if (ch == 133) {
                                            pos3 = pos + 1;
                                            start = pos;
                                            length2 = pos3 + 1;
                                        } else {
                                            in.incrLineNumber(1, pos);
                                            start = pos;
                                            pos5 = pos + 1;
                                            length2 = pos5;
                                        }
                                    }
                                    in.incrLineNumber(1, pos3);
                                    pos = pos3;
                                } else {
                                    out.linefeedFromParser();
                                    state = 28;
                                }
                            } else if (ch == 133 || ch == 8232) {
                                int length3 = pos - length2;
                                in.pos = pos - 1;
                                if (length3 > 0) {
                                    out.textFromParser(buffer, start, length3);
                                }
                                out.linefeedFromParser();
                                in.incrLineNumber(1, pos);
                                length2 = pos + 1;
                                start = pos;
                            } else if (ch == '\n') {
                                in.incrLineNumber(1, pos);
                            }
                            if (pos == limit) {
                                length2--;
                                pos2 = pos;
                            } else {
                                pos5 = pos + 1;
                                ch = buffer[pos];
                            }
                        }
                    }
                    length = pos2 - length2;
                    if (length > 0) {
                        in.pos = pos2;
                        out.textFromParser(buffer, start, length);
                    }
                    start = buffer.length;
                    pos = pos2;
                    if (pos < limit) {
                    }
                    break;
                case 2:
                    in.pos = pos5 - length;
                    out.emitStartElement(buffer, start, length);
                    state = 12;
                    start = limit;
                case 3:
                case 5:
                case 7:
                case 9:
                case 17:
                case 24:
                case 33:
                    length = start + 1;
                    while (true) {
                        pos = pos5;
                        if ((ch >= 'a' && ch <= 'z') || ((ch >= 'A' && ch <= 'Z') || ch == '_' || ch == ':' || ((ch >= 192 && (ch <= 767 || (ch >= 880 && ((ch <= 8191 && ch != 894) || (ch >= 8204 && (ch <= 8205 || ((ch >= 8304 && ch <= 8591) || ((ch >= 11264 && ch <= 12271) || ((ch >= 12289 && ch <= 55295) || (ch >= 63744 && ch <= 65533)))))))))) || ((pos > length && ch >= '0' && ch <= '9') || ch == '.' || ch == '-' || ch == 183 || (ch > 768 && (ch <= 879 || (ch >= 8255 && ch <= 8256))))))) {
                            if (pos < limit) {
                                pos5 = pos + 1;
                                ch = buffer[pos];
                            }
                        }
                    }
                    if (pos < limit) {
                    }
                    break;
                case 4:
                    in.pos = pos5;
                    out.emitEndElement(buffer, start, length);
                    start = limit;
                    state = 29;
                case 6:
                    in.pos = pos5;
                    if (ch != ';') {
                        out.error('w', "missing ';'");
                    }
                    out.emitEntityReference(buffer, start, length);
                    start = limit;
                    state = 1;
                    pos = pos5;
                    if (pos < limit) {
                    }
                    break;
                case 8:
                    if (ch != ' ' && ch != '\t' && ch != '\r' && ch != '\n' && ch != 133) {
                        if (ch == 8232) {
                            pos = pos5;
                        } else {
                            in.pos = pos5 - length;
                            out.emitStartAttribute(buffer, start, length);
                            start = limit;
                            if (ch == '=') {
                                state = 11;
                                pos = pos5;
                            } else {
                                out.emitEndAttributes();
                                message = "missing or misplaced '=' after attribute name";
                                state = 36;
                            }
                        }
                        if (pos < limit) {
                        }
                    }
                    pos = pos5;
                    if (pos < limit) {
                    }
                    break;
                case 10:
                    terminator = '<';
                    continue_state = 14;
                    if (ch == '/') {
                        in.pos = pos5;
                        out.emitEndAttributes();
                        out.emitEndElement(null, 0, 0);
                        state = 27;
                        pos = pos5;
                    } else if (ch == '>') {
                        in.pos = pos5;
                        out.emitEndAttributes();
                        state = 1;
                        pos = pos5;
                    } else {
                        start = pos5 - 1;
                        state = 9;
                    }
                    if (pos < limit) {
                    }
                    break;
                case 11:
                    if (ch == '\'' || ch == '\"') {
                        terminator = ch;
                        continue_state = 12;
                        state = 1;
                        pos = pos5;
                    } else {
                        if (ch != ' ' && ch != '\t' && ch != '\r' && ch != '\n' && ch != 133) {
                            if (ch == 8232) {
                                pos = pos5;
                            } else {
                                message = "missing or unquoted attribute value";
                                state = 36;
                            }
                        }
                        pos = pos5;
                    }
                    if (pos < limit) {
                    }
                    break;
                case 12:
                case 15:
                case 23:
                case 29:
                case 32:
                    if (ch != ' ') {
                        if (ch == '\t') {
                            pos = pos5;
                        } else if (ch == '\n' || ch == '\r' || ch == 133 || ch == 8232) {
                            in.incrLineNumber(1, pos5);
                            pos = pos5;
                        } else {
                            state -= 2;
                        }
                        if (pos < limit) {
                        }
                    } else {
                        pos = pos5;
                        if (pos < limit) {
                        }
                    }
                    break;
                case 13:
                    state = 17;
                    start = pos5 - 1;
                case 14:
                    if (ch == '/') {
                        state = 19;
                        pos = pos5;
                    } else if (ch == '?') {
                        start = pos5;
                        state = 24;
                        pos = pos5;
                    } else if (ch == '!') {
                        state = 20;
                        start = pos5;
                        pos = pos5;
                    } else {
                        start = pos5 - 1;
                        state = 3;
                    }
                    if (pos < limit) {
                    }
                    break;
                case 16:
                    if (dstart < 0) {
                        int dstart2 = pos5 - 1;
                        dstart = (dstart2 - start) << 1;
                        terminator = 0;
                    }
                    while (true) {
                        pos = pos5;
                        if (ch == '\'' || ch == '\"') {
                            if (terminator == 0) {
                                terminator = ch;
                            } else if (terminator == ch) {
                                terminator = 0;
                            }
                        } else if (terminator == 0) {
                            if (ch == '[') {
                                dstart |= 1;
                            } else if (ch == ']') {
                                dstart &= -2;
                            } else if (ch == '>' && (dstart & 1) == 0) {
                                in.pos = pos;
                                int dstart3 = (dstart >> 1) + start;
                                out.emitDoctypeDecl(buffer, start, length, dstart3, (pos - 1) - dstart3);
                                terminator = '<';
                                start = limit;
                                dstart = -1;
                                state = 1;
                            }
                        }
                        if (pos < limit) {
                            pos5 = pos + 1;
                            ch = buffer[pos];
                        }
                    }
                    if (pos < limit) {
                    }
                    break;
                case 18:
                case 22:
                default:
                    pos = pos5;
                    if (pos < limit) {
                    }
                    break;
                case 19:
                    start = pos5 - 1;
                    state = 5;
                case 20:
                    while (true) {
                        pos = pos5;
                        if (ch == '>') {
                            length = (pos - 1) - start;
                            if (length >= 4 && buffer[start] == '-' && buffer[start + 1] == '-') {
                                if (buffer[pos - 2] == '-' && buffer[pos - 3] == '-') {
                                    in.pos = pos;
                                    out.commentFromParser(buffer, start + 2, length - 4);
                                }
                                if (pos < limit) {
                                }
                            } else if (length >= 6 && buffer[start] == '[' && buffer[start + 1] == 'C' && buffer[start + 2] == 'D' && buffer[start + 3] == 'A' && buffer[start + 4] == 'T' && buffer[start + 5] == 'A' && buffer[start + 6] == '[') {
                                if (buffer[pos - 2] == ']' && buffer[pos - 3] == ']') {
                                    in.pos = pos;
                                    out.writeCDATA(buffer, start + 7, (pos - 10) - start);
                                }
                                if (pos < limit) {
                                    pos5 = pos + 1;
                                    ch = buffer[pos];
                                }
                            }
                        } else {
                            if (pos == start + 7 && buffer[start] == 'D' && buffer[start + 1] == 'O' && buffer[start + 2] == 'C' && buffer[start + 3] == 'T' && buffer[start + 4] == 'Y' && buffer[start + 5] == 'P' && ch == 'E') {
                                start = limit;
                                state = 15;
                            }
                            if (pos < limit) {
                            }
                        }
                    }
                    start = limit;
                    state = 1;
                    if (pos < limit) {
                    }
                    break;
                case 21:
                case 30:
                    if (dstart < 0) {
                        dstart = pos5 - 1;
                    }
                    while (true) {
                        pos = pos5;
                        if (ch == '>') {
                            int end = pos - 2;
                            if (buffer[end] == '?' && end >= dstart) {
                                in.pos = pos;
                                if (length == 3 && buffer[start] == 'x' && buffer[start + 1] == 'm' && buffer[start + 2] == 'l') {
                                    if (state == 30) {
                                        if (end <= dstart + 7 || buffer[dstart] != 'v' || buffer[dstart + 1] != 'e' || buffer[dstart + 2] != 'r' || buffer[dstart + 3] != 's' || buffer[dstart + 4] != 'i' || buffer[dstart + 5] != 'o' || buffer[dstart + 6] != 'n') {
                                            pos5 = dstart;
                                            message = "xml declaration without version";
                                            state = 36;
                                        } else {
                                            dstart += 7;
                                            ch = buffer[dstart];
                                            while (Character.isWhitespace(ch) && (dstart = dstart + 1) < end) {
                                                ch = buffer[dstart];
                                            }
                                            if (ch != '=') {
                                                state = 35;
                                                pos5 = pos;
                                            } else {
                                                dstart++;
                                                ch = buffer[dstart];
                                                while (Character.isWhitespace(ch) && (dstart = dstart + 1) < end) {
                                                    ch = buffer[dstart];
                                                }
                                                if (ch != '\'' && ch != '\"') {
                                                    state = 35;
                                                    pos5 = pos;
                                                } else {
                                                    char quote = ch;
                                                    dstart++;
                                                    int i = dstart;
                                                    while (true) {
                                                        if (i == end) {
                                                            state = 35;
                                                            pos5 = pos;
                                                        } else {
                                                            ch = buffer[i];
                                                            if (ch != quote) {
                                                                i++;
                                                            } else if ((i != dstart + 3 || buffer[dstart] != '1' || buffer[dstart + 1] != '.' || (ch = buffer[dstart + 2]) != '0') && ch != '1') {
                                                                state = 35;
                                                                pos5 = pos;
                                                            } else {
                                                                dstart = i + 1;
                                                                while (dstart < end && Character.isWhitespace(buffer[dstart])) {
                                                                    dstart++;
                                                                }
                                                                if (end > dstart + 7 && buffer[dstart] == 'e' && buffer[dstart + 1] == 'n' && buffer[dstart + 2] == 'c' && buffer[dstart + 3] == 'o' && buffer[dstart + 4] == 'd' && buffer[dstart + 5] == 'i' && buffer[dstart + 6] == 'n' && buffer[dstart + 7] == 'g') {
                                                                    dstart += 8;
                                                                    ch = buffer[dstart];
                                                                    while (Character.isWhitespace(ch) && (dstart = dstart + 1) < end) {
                                                                        ch = buffer[dstart];
                                                                    }
                                                                    if (ch != '=') {
                                                                        message = BAD_ENCODING_SYNTAX;
                                                                        state = 36;
                                                                        pos5 = pos;
                                                                    } else {
                                                                        dstart++;
                                                                        ch = buffer[dstart];
                                                                        while (Character.isWhitespace(ch) && (dstart = dstart + 1) < end) {
                                                                            ch = buffer[dstart];
                                                                        }
                                                                        if (ch != '\'' && ch != '\"') {
                                                                            message = BAD_ENCODING_SYNTAX;
                                                                            state = 36;
                                                                            pos5 = pos;
                                                                        } else {
                                                                            char quote2 = ch;
                                                                            dstart++;
                                                                            for (int i2 = dstart; i2 != end; i2++) {
                                                                                ch = buffer[i2];
                                                                                if (ch == quote2) {
                                                                                    String encoding = new String(buffer, dstart, i2 - dstart);
                                                                                    if (in instanceof LineInputStreamReader) {
                                                                                        ((LineInputStreamReader) in).setCharset(encoding);
                                                                                    }
                                                                                    dstart = i2 + 1;
                                                                                    while (dstart < end && Character.isWhitespace(buffer[dstart])) {
                                                                                        dstart++;
                                                                                    }
                                                                                }
                                                                            }
                                                                            message = BAD_ENCODING_SYNTAX;
                                                                            state = 36;
                                                                            pos5 = pos;
                                                                        }
                                                                    }
                                                                }
                                                                if (end > dstart + 9 && buffer[dstart] == 's' && buffer[dstart + 1] == 't' && buffer[dstart + 2] == 'a' && buffer[dstart + 3] == 'n' && buffer[dstart + 4] == 'd' && buffer[dstart + 5] == 'a' && buffer[dstart + 6] == 'l' && buffer[dstart + 7] == 'o' && buffer[dstart + 8] == 'n' && buffer[dstart + 9] == 'e') {
                                                                    dstart += 10;
                                                                    ch = buffer[dstart];
                                                                    while (Character.isWhitespace(ch) && (dstart = dstart + 1) < end) {
                                                                        ch = buffer[dstart];
                                                                    }
                                                                    if (ch != '=') {
                                                                        message = BAD_STANDALONE_SYNTAX;
                                                                        state = 36;
                                                                        pos5 = pos;
                                                                    } else {
                                                                        dstart++;
                                                                        ch = buffer[dstart];
                                                                        while (Character.isWhitespace(ch) && (dstart = dstart + 1) < end) {
                                                                            ch = buffer[dstart];
                                                                        }
                                                                        if (ch != '\'' && ch != '\"') {
                                                                            message = BAD_STANDALONE_SYNTAX;
                                                                            state = 36;
                                                                            pos5 = pos;
                                                                        } else {
                                                                            char quote3 = ch;
                                                                            dstart++;
                                                                            int i3 = dstart;
                                                                            while (true) {
                                                                                if (i3 == end) {
                                                                                    message = BAD_STANDALONE_SYNTAX;
                                                                                    state = 36;
                                                                                    pos5 = pos;
                                                                                } else {
                                                                                    ch = buffer[i3];
                                                                                    if (ch != quote3) {
                                                                                        i3++;
                                                                                    } else if ((i3 != dstart + 3 || buffer[dstart] != 'y' || buffer[dstart + 1] != 'e' || buffer[dstart + 2] != 's') && (i3 != dstart + 2 || buffer[dstart] != 'n' || buffer[dstart + 1] != 'o')) {
                                                                                        message = BAD_STANDALONE_SYNTAX;
                                                                                        state = 36;
                                                                                        pos5 = pos;
                                                                                    } else {
                                                                                        dstart = i3 + 1;
                                                                                        while (dstart < end && Character.isWhitespace(buffer[dstart])) {
                                                                                            dstart++;
                                                                                        }
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    } else {
                                        message = "<?xml must be at start of file";
                                        state = 36;
                                        pos5 = pos;
                                    }
                                } else {
                                    out.processingInstructionFromParser(buffer, start, length, dstart, end - dstart);
                                }
                                start = limit;
                                dstart = -1;
                                state = 1;
                            }
                        }
                        if (pos < limit) {
                            pos5 = pos + 1;
                            ch = buffer[pos];
                        }
                    }
                    if (pos < limit) {
                    }
                    break;
                case 25:
                    if (ch == '#') {
                        state = 26;
                        start = pos5;
                        length = 0;
                        dstart = 0;
                        pos = pos5;
                        if (pos < limit) {
                        }
                    } else {
                        start = pos5 - 1;
                        state = 7;
                    }
                    break;
                case 26:
                    while (true) {
                        pos = pos5;
                        if (ch == ';') {
                            in.pos = pos;
                            out.emitCharacterReference(length, buffer, start, (pos - 1) - start);
                            state = 1;
                        } else {
                            if (ch == 'x' && dstart == 0) {
                                dstart = 16;
                            } else if (length >= 134217728) {
                                break;
                            } else {
                                int base = dstart == 0 ? 10 : dstart;
                                int digit = Character.digit(ch, base);
                                if (digit < 0) {
                                    break;
                                } else {
                                    length = (length * base) + digit;
                                }
                            }
                            if (pos < limit) {
                                pos5 = pos + 1;
                                ch = buffer[pos];
                            }
                        }
                    }
                    if (pos < limit) {
                    }
                    break;
                case 27:
                    if (ch != '>') {
                        message = "missing '>'";
                        state = 36;
                    } else {
                        state = 1;
                        pos = pos5;
                        if (pos < limit) {
                        }
                    }
                    break;
                case 28:
                    state = 1;
                    if ((ch == 133) | (ch == '\n')) {
                        in.incrLineNumber(1, pos5);
                        pos = pos5;
                        if (pos < limit) {
                        }
                    } else {
                        in.incrLineNumber(1, pos5 - 1);
                    }
                    break;
                case 31:
                    if (ch == '<') {
                        state = 34;
                        pos = pos5;
                        if (pos < limit) {
                        }
                    } else {
                        state = 1;
                    }
                    break;
                case 34:
                    if (ch == '?') {
                        start = pos5;
                        state = 33;
                        pos = pos5;
                        if (pos < limit) {
                        }
                    } else {
                        state = 14;
                    }
                    break;
                case 35:
                    pos5 = dstart;
                    message = "invalid xml version specifier";
                    in.pos = pos5;
                    out.error('e', message);
                    do {
                        pos4 = pos5;
                        if (pos4 >= limit) {
                            pos5 = pos4 + 1;
                            ch = buffer[pos4];
                        } else {
                            return;
                        }
                    } while (ch != '>');
                    state = 1;
                    pos = pos5;
                    if (pos < limit) {
                    }
                    break;
                case 36:
                    in.pos = pos5;
                    out.error('e', message);
                    do {
                        pos4 = pos5;
                        if (pos4 >= limit) {
                        }
                    } while (ch != '>');
                    state = 1;
                    pos = pos5;
                    if (pos < limit) {
                    }
                    break;
                case 37:
                    in.pos = pos5;
                    out.error('f', "unexpected end-of-file");
                    return;
            }
        }
    }
}
