package gnu.xquery.util;

import gnu.bytecode.Access;
import gnu.bytecode.Type;
import gnu.expr.ApplyExp;
import gnu.expr.Compilation;
import gnu.expr.Expression;
import gnu.expr.Inlineable;
import gnu.expr.Target;
import gnu.kawa.functions.AddOp;
import gnu.kawa.functions.Arithmetic;
import gnu.kawa.functions.MultiplyOp;
import gnu.kawa.xml.KNode;
import gnu.kawa.xml.UntypedAtomic;
import gnu.kawa.xml.XDataType;
import gnu.mapping.Procedure;
import gnu.mapping.Procedure1or2;
import gnu.mapping.Values;
import gnu.math.DFloNum;
import gnu.math.Duration;
import gnu.math.IntNum;
import gnu.math.Numeric;
import gnu.math.RealNum;
import gnu.math.Unit;
import gnu.xml.TextUtils;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.MathContext;
/* loaded from: classes.dex */
public class ArithOp extends Procedure1or2 implements Inlineable {
    char op;
    static final BigInteger TEN = BigInteger.valueOf(10);
    public static final ArithOp add = new ArithOp("+", '+', 2);
    public static final ArithOp sub = new ArithOp("-", '-', 2);
    public static final ArithOp mul = new ArithOp("*", '*', 2);
    public static final ArithOp div = new ArithOp("div", 'd', 2);
    public static final ArithOp idiv = new ArithOp("idiv", 'i', 2);
    public static final ArithOp mod = new ArithOp("mod", 'm', 2);
    public static final ArithOp plus = new ArithOp("+", 'P', 1);
    public static final ArithOp minus = new ArithOp("-", Access.METHOD_CONTEXT, 1);

    ArithOp(String name, char op, int nargs) {
        super(name);
        setProperty(Procedure.validateApplyKey, "gnu.xquery.util.CompileMisc:validateArithOp");
        this.op = op;
    }

    @Override // gnu.mapping.Procedure1or2, gnu.mapping.Procedure
    public Object apply1(Object arg1) throws Throwable {
        if (arg1 == Values.empty || arg1 == null) {
            return arg1;
        }
        if ((arg1 instanceof KNode) || (arg1 instanceof UntypedAtomic)) {
            arg1 = XDataType.doubleType.valueOf(TextUtils.stringValue(arg1));
        }
        switch (this.op) {
            case 'M':
                int code1 = Arithmetic.classifyValue(arg1);
                switch (code1) {
                    case 7:
                        return XDataType.makeFloat(-Arithmetic.asFloat(arg1));
                    case 8:
                        return XDataType.makeDouble(-Arithmetic.asDouble(arg1));
                    default:
                        if (arg1 instanceof Numeric) {
                            return ((Numeric) arg1).neg();
                        }
                        return AddOp.apply2(-1, IntNum.zero(), arg1);
                }
            case 'N':
            case 'O':
            default:
                throw new UnsupportedOperationException(getName());
            case 'P':
                return AddOp.apply2(1, IntNum.zero(), arg1);
        }
    }

    public static BigDecimal div(BigDecimal d1, BigDecimal d2) {
        return d1.divide(d2, MathContext.DECIMAL128);
    }

    /* JADX WARN: Removed duplicated region for block: B:67:0x01be  */
    /* JADX WARN: Removed duplicated region for block: B:68:0x01cc  */
    @Override // gnu.mapping.Procedure1or2, gnu.mapping.Procedure
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public Object apply2(Object arg1, Object arg2) throws Throwable {
        if (arg1 == Values.empty || arg1 == null) {
            return arg1;
        }
        if (arg2 == Values.empty || arg2 == null) {
            return arg2;
        }
        if ((arg1 instanceof KNode) || (arg1 instanceof UntypedAtomic)) {
            arg1 = XDataType.doubleType.valueOf(TextUtils.stringValue(arg1));
        }
        if ((arg2 instanceof KNode) || (arg2 instanceof UntypedAtomic)) {
            arg2 = XDataType.doubleType.valueOf(TextUtils.stringValue(arg2));
        }
        switch (this.op) {
            case '*':
                return MultiplyOp.$St.apply2(arg1, arg2);
            case '+':
                return AddOp.apply2(1, arg1, arg2);
            case ',':
            default:
                int code1 = Arithmetic.classifyValue(arg1);
                int code2 = Arithmetic.classifyValue(arg2);
                int code = code1 < code2 ? code2 : code1;
                switch (this.op) {
                    case 'd':
                        if (code1 >= 0 && code2 >= 0) {
                            if (code <= 6) {
                                BigDecimal d1 = (BigDecimal) XDataType.decimalType.cast(arg1);
                                BigDecimal d2 = (BigDecimal) XDataType.decimalType.cast(arg2);
                                return div(d1, d2);
                            } else if (code == 7) {
                                return new Float(((Number) arg1).floatValue() / ((Number) arg2).floatValue());
                            } else {
                                if (code == 8) {
                                    return new Double(((Number) arg1).doubleValue() / ((Number) arg2).doubleValue());
                                }
                                if ((arg1 instanceof Duration) && (arg2 instanceof Duration)) {
                                    Duration dur1 = (Duration) arg1;
                                    Duration dur2 = (Duration) arg2;
                                    if (dur1.unit() == Unit.second && dur2.unit() == Unit.second) {
                                        long s1 = dur1.getTotalSeconds();
                                        long s2 = dur2.getTotalSeconds();
                                        int n1 = dur1.getNanoSecondsOnly();
                                        int n2 = dur2.getNanoSecondsOnly();
                                        BigDecimal sec1 = TimeUtils.secondsBigDecimalFromDuration(s1, n1);
                                        BigDecimal sec2 = TimeUtils.secondsBigDecimalFromDuration(s2, n2);
                                        return div(sec1, sec2);
                                    } else if (dur1.unit() == Unit.month && dur2.unit() == Unit.month) {
                                        BigDecimal m1 = BigDecimal.valueOf(dur1.getTotalMonths());
                                        BigDecimal m2 = BigDecimal.valueOf(dur2.getTotalMonths());
                                        return div(m1, m2);
                                    } else {
                                        throw new ArithmeticException("divide of incompatible durations");
                                    }
                                }
                                if (code >= 0) {
                                    return Arithmetic.asNumeric(arg1).div(Arithmetic.asNumeric(arg2));
                                }
                                if (code1 >= 0 && code2 >= 0) {
                                    if (code > 4) {
                                        IntNum i1 = Arithmetic.asIntNum(arg1);
                                        IntNum i2 = Arithmetic.asIntNum(arg2);
                                        return IntNum.quotient(i1, i2);
                                    } else if (code <= 6) {
                                        BigDecimal d12 = (BigDecimal) XDataType.decimalType.cast(arg1);
                                        BigDecimal d22 = (BigDecimal) XDataType.decimalType.cast(arg2);
                                        return Arithmetic.asIntNum(d12.divide(d22, 0, 1));
                                    } else if (code <= 7) {
                                        float f = ((Number) arg1).floatValue() / ((Number) arg2).floatValue();
                                        return RealNum.toExactInt(f, 3);
                                    } else {
                                        return RealNum.toExactInt(((Number) arg1).doubleValue() / ((Number) arg2).doubleValue(), 3);
                                    }
                                }
                            }
                        }
                        throw new UnsupportedOperationException(getName());
                    case 'i':
                        if (code1 >= 0) {
                            if (code > 4) {
                            }
                        }
                        throw new UnsupportedOperationException(getName());
                    case 'm':
                        if (code1 >= 0 && code2 >= 0) {
                            if (code <= 4) {
                                IntNum i12 = Arithmetic.asIntNum(arg1);
                                IntNum i22 = Arithmetic.asIntNum(arg2);
                                return IntNum.remainder(i12, i22);
                            } else if (code <= 6) {
                                return sub.apply2(arg1, mul.apply2(idiv.apply2(arg1, arg2), arg2));
                            } else if (code <= 7) {
                                float f1 = Arithmetic.asFloat(arg1);
                                float f2 = Arithmetic.asFloat(arg2);
                                return XDataType.makeFloat(f1 % f2);
                            } else if (code <= 9) {
                                double d13 = Arithmetic.asDouble(arg1);
                                double d23 = Arithmetic.asDouble(arg2);
                                double d = d13 % d23;
                                if (code == 9) {
                                    return DFloNum.make(d);
                                }
                                return XDataType.makeDouble(d);
                            }
                        }
                        throw new UnsupportedOperationException(getName());
                    default:
                        throw new UnsupportedOperationException(getName());
                }
            case '-':
                return AddOp.apply2(-1, arg1, arg2);
        }
    }

    @Override // gnu.expr.Inlineable
    public void compile(ApplyExp exp, Compilation comp, Target target) {
        ApplyExp.compile(exp, comp, target);
    }

    @Override // gnu.mapping.Procedure
    public Type getReturnType(Expression[] args) {
        return Type.pointer_type;
    }
}
