package kawa.lang;

import gnu.bytecode.ClassType;
import gnu.bytecode.Field;
import gnu.bytecode.Type;
import gnu.lists.LList;
import gnu.lists.Pair;
import gnu.mapping.ProcedureN;
import gnu.mapping.WrappedException;
import gnu.mapping.WrongArguments;
/* loaded from: classes.dex */
public class RecordConstructor extends ProcedureN {
    Field[] fields;
    ClassType type;

    public RecordConstructor(ClassType type, Field[] fields) {
        this.type = type;
        this.fields = fields;
    }

    public RecordConstructor(Class clas, Field[] fields) {
        this((ClassType) Type.make(clas), fields);
    }

    public RecordConstructor(Class clas) {
        init((ClassType) Type.make(clas));
    }

    public RecordConstructor(ClassType type) {
        init(type);
    }

    private void init(ClassType type) {
        int i;
        this.type = type;
        Field list = type.getFields();
        int count = 0;
        for (Field fld = list; fld != null; fld = fld.getNext()) {
            if ((fld.getModifiers() & 9) == 1) {
                count++;
            }
        }
        this.fields = new Field[count];
        Field fld2 = list;
        int i2 = 0;
        while (fld2 != null) {
            if ((fld2.getModifiers() & 9) == 1) {
                i = i2 + 1;
                this.fields[i2] = fld2;
            } else {
                i = i2;
            }
            fld2 = fld2.getNext();
            i2 = i;
        }
    }

    public RecordConstructor(Class clas, Object fieldsList) {
        this((ClassType) Type.make(clas), fieldsList);
    }

    /* JADX WARN: Code restructure failed: missing block: B:14:0x0056, code lost:
        r9.fields[r2] = r0;
        r11 = r5.getCdr();
        r2 = r2 + 1;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public RecordConstructor(ClassType type, Object fieldsList) {
        this.type = type;
        if (fieldsList == null) {
            init(type);
            return;
        }
        int nfields = LList.listLength(fieldsList, false);
        this.fields = new Field[nfields];
        Field list = type.getFields();
        int i = 0;
        while (i < nfields) {
            Pair pair = (Pair) fieldsList;
            String fname = pair.getCar().toString();
            Field fld = list;
            while (fld != null) {
                if (fld.getSourceName() == fname) {
                    break;
                }
                fld = fld.getNext();
            }
            throw new RuntimeException("no such field " + fname + " in " + type.getName());
        }
    }

    @Override // gnu.mapping.Procedure
    public int numArgs() {
        int nargs = this.fields.length;
        return (nargs << 12) | nargs;
    }

    @Override // gnu.mapping.PropertySet, gnu.mapping.Named
    public String getName() {
        return this.type.getName() + " constructor";
    }

    @Override // gnu.mapping.ProcedureN, gnu.mapping.Procedure
    public Object applyN(Object[] args) {
        try {
            Object obj = this.type.getReflectClass().newInstance();
            if (args.length != this.fields.length) {
                throw new WrongArguments(this, args.length);
            }
            for (int i = 0; i < args.length; i++) {
                Field fld = this.fields[i];
                try {
                    fld.getReflectField().set(obj, args[i]);
                } catch (Exception ex) {
                    throw new WrappedException("illegal access for field " + fld.getName(), ex);
                }
            }
            return obj;
        } catch (IllegalAccessException ex2) {
            throw new GenericError(ex2.toString());
        } catch (InstantiationException ex3) {
            throw new GenericError(ex3.toString());
        }
    }
}
