package kawa.standard;

import android.support.v4.media.session.PlaybackStateCompat;
import gnu.bytecode.ClassType;
import gnu.bytecode.Field;
import gnu.bytecode.Method;
import gnu.bytecode.Type;
import gnu.expr.ApplyExp;
import gnu.expr.Compilation;
import gnu.expr.Declaration;
import gnu.expr.Expression;
import gnu.expr.Language;
import gnu.expr.ModuleExp;
import gnu.expr.ModuleInfo;
import gnu.expr.ModuleManager;
import gnu.expr.QuoteExp;
import gnu.expr.ReferenceExp;
import gnu.expr.ScopeExp;
import gnu.expr.SetExp;
import gnu.kawa.lispexpr.LispLanguage;
import gnu.kawa.reflect.Invoke;
import gnu.kawa.reflect.SlotGet;
import gnu.lists.LList;
import gnu.lists.Pair;
import gnu.mapping.InPort;
import gnu.mapping.Procedure;
import gnu.mapping.SimpleSymbol;
import gnu.mapping.Symbol;
import gnu.text.Path;
import gnu.text.SourceMessages;
import gnu.text.SyntaxException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import kawa.lang.Syntax;
import kawa.lang.Translator;
/* loaded from: classes.dex */
public class require extends Syntax {
    private static final String SLIB_PREFIX = "gnu.kawa.slib.";
    static Hashtable featureMap;
    public static final require require = new require();

    static {
        require.setName("require");
        featureMap = new Hashtable();
        map("generic-write", "gnu.kawa.slib.genwrite");
        map("pretty-print", "gnu.kawa.slib.pp");
        map("pprint-file", "gnu.kawa.slib.ppfile");
        map("printf", "gnu.kawa.slib.printf");
        map("xml", "gnu.kawa.slib.XML");
        map("readtable", "gnu.kawa.slib.readtable");
        map("srfi-10", "gnu.kawa.slib.readtable");
        map("http", "gnu.kawa.servlet.HTTP");
        map("servlets", "gnu.kawa.servlet.servlets");
        map("srfi-1", "gnu.kawa.slib.srfi1");
        map("list-lib", "gnu.kawa.slib.srfi1");
        map("srfi-2", "gnu.kawa.slib.srfi2");
        map("and-let*", "gnu.kawa.slib.srfi2");
        map("srfi-13", "gnu.kawa.slib.srfi13");
        map("string-lib", "gnu.kawa.slib.srfi13");
        map("srfi-34", "gnu.kawa.slib.srfi34");
        map("srfi-35", "gnu.kawa.slib.conditions");
        map("condition", "gnu.kawa.slib.conditions");
        map("conditions", "gnu.kawa.slib.conditions");
        map("srfi-37", "gnu.kawa.slib.srfi37");
        map("args-fold", "gnu.kawa.slib.srfi37");
        map("srfi-64", "gnu.kawa.slib.testing");
        map("testing", "gnu.kawa.slib.testing");
        map("srfi-69", "gnu.kawa.slib.srfi69");
        map("hash-table", "gnu.kawa.slib.srfi69");
        map("basic-hash-tables", "gnu.kawa.slib.srfi69");
        map("srfi-95", "kawa.lib.srfi95");
        map("sorting-and-merging", "kawa.lib.srfi95");
        map("regex", "kawa.lib.kawa.regex");
        map("pregexp", "gnu.kawa.slib.pregexp");
        map("gui", "gnu.kawa.slib.gui");
        map("swing-gui", "gnu.kawa.slib.swing");
        map("android-defs", "gnu.kawa.android.defs");
        map("syntax-utils", "gnu.kawa.slib.syntaxutils");
    }

    static void map(String featureName, String className) {
        featureMap.put(featureName, className);
    }

    public static String mapFeature(String featureName) {
        return (String) featureMap.get(featureName);
    }

    public static Object find(String typeName) {
        return ModuleManager.getInstance().findWithClassName(typeName).getInstance();
    }

    /* JADX WARN: Removed duplicated region for block: B:23:0x0099  */
    /* JADX WARN: Removed duplicated region for block: B:48:0x014f A[EXC_TOP_SPLITTER, SYNTHETIC] */
    @Override // kawa.lang.Syntax
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public boolean scanForDefinitions(Pair st, Vector forms, ScopeExp defs, Translator tr) {
        if (tr.getState() == 1) {
            tr.setState(2);
            tr.pendingForm = st;
            return true;
        }
        Pair args = (Pair) st.getCdr();
        Object name = args.getCar();
        Type type = null;
        if (name instanceof Pair) {
            Pair p = (Pair) name;
            if (tr.matches(p.getCar(), LispLanguage.quote_sym)) {
                Object name2 = p.getCdr();
                if (name2 instanceof Pair) {
                    Pair p2 = (Pair) name2;
                    if (p2.getCdr() == LList.Empty && (p2.getCar() instanceof Symbol)) {
                        Object name3 = mapFeature(p2.getCar().toString());
                        if (name3 == null) {
                            tr.error('e', "unknown feature name '" + p2.getCar() + "' for 'require'");
                            return false;
                        }
                        type = ClassType.make((String) name3);
                        if (type instanceof ClassType) {
                            tr.error('e', "invalid specifier for 'require'");
                            return false;
                        }
                        try {
                            ModuleInfo minfo = ModuleInfo.find((ClassType) type);
                            importDefinitions(null, minfo, null, forms, defs, tr);
                            return true;
                        } catch (Exception e) {
                            tr.error('e', "unknown class " + type.getName());
                            return false;
                        }
                    }
                }
                tr.error('e', "invalid quoted symbol for 'require'");
                return false;
            }
        }
        if (name instanceof CharSequence) {
            String sourceName = name.toString();
            ModuleInfo info = lookupModuleFromSourcePath(sourceName, defs);
            if (info == null) {
                tr.error('e', "malformed URL: " + sourceName);
                return false;
            }
            return importDefinitions(null, info, null, forms, defs, tr);
        }
        if ((name instanceof Symbol) && !tr.selfEvaluatingSymbol(name)) {
            type = tr.getLanguage().getTypeFor(tr.rewrite(name, false));
            if ((type instanceof ClassType) && (args.getCdr() instanceof Pair)) {
                Object name4 = ((Pair) args.getCdr()).getCar();
                if (name4 instanceof CharSequence) {
                    String sourceName2 = name4.toString();
                    ModuleInfo info2 = lookupModuleFromSourcePath(sourceName2, defs);
                    if (info2 == null) {
                        tr.error('e', "malformed URL: " + sourceName2);
                        return false;
                    }
                    return importDefinitions(type.getName(), info2, null, forms, defs, tr);
                }
            }
        }
        if (type instanceof ClassType) {
        }
    }

    public static ModuleInfo lookupModuleFromSourcePath(String sourceName, ScopeExp defs) {
        ModuleManager manager = ModuleManager.getInstance();
        String baseName = defs.getFileName();
        if (baseName != null) {
            sourceName = Path.valueOf(baseName).resolve(sourceName).toString();
        }
        return manager.findWithSourcePath(sourceName);
    }

    /* JADX WARN: Removed duplicated region for block: B:105:0x03ea  */
    /* JADX WARN: Removed duplicated region for block: B:76:0x0304  */
    /* JADX WARN: Removed duplicated region for block: B:81:0x032a  */
    /* JADX WARN: Removed duplicated region for block: B:84:0x0338  */
    /* JADX WARN: Removed duplicated region for block: B:86:0x0341  */
    /* JADX WARN: Removed duplicated region for block: B:89:0x0364  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static boolean importDefinitions(String className, ModuleInfo info, Procedure renamer, Vector forms, ScopeExp defs, Compilation tr) {
        Object obj;
        Declaration adecl;
        ModuleManager manager = ModuleManager.getInstance();
        if ((info.getState() & 1) == 0 && info.getCompilation() == null) {
            long now = System.currentTimeMillis();
            if (!info.checkCurrent(manager, now)) {
                SourceMessages messages = tr.getMessages();
                Language language = Language.getDefaultLanguage();
                try {
                    InPort fstream = InPort.openFile(info.getSourceAbsPath());
                    info.clearClass();
                    info.setClassName(className);
                    int options = 8;
                    if (tr.immediate) {
                        options = 8 | 1;
                    }
                    Compilation comp = language.parse(fstream, messages, options, info);
                    info.setClassName(comp.getModule().classFor(comp).getName());
                } catch (SyntaxException ex) {
                    if (ex.getMessages() != messages) {
                        throw new RuntimeException("confussing syntax error: " + ex);
                    }
                    return false;
                } catch (FileNotFoundException ex2) {
                    tr.error('e', "not found: " + ex2.getMessage());
                    return false;
                } catch (IOException ex3) {
                    tr.error('e', "caught " + ex3);
                    return false;
                }
            }
        }
        if (tr.minfo != null && tr.getState() < 4) {
            tr.minfo.addDependency(info);
            if (!info.loadEager(12) && info.getState() < 6) {
                tr.pushPendingImport(info, defs, forms.size());
                return true;
            }
        }
        ClassType type = info.getClassType();
        String tname = type.getName();
        boolean sharedModule = tr.sharedModuleDefs();
        boolean isRunnable = info.getState() < 6 ? info.getCompilation().makeRunnable() : type.isSubtype(Compilation.typeRunnable);
        Declaration decl = null;
        ClassType thisType = ClassType.make("kawa.standard.require");
        Expression[] args = {new QuoteExp(tname)};
        Expression dofind = Invoke.makeInvokeStatic(thisType, "find", args);
        Field instanceField = null;
        Language language2 = tr.getLanguage();
        dofind.setLine(tr);
        int formsStart = forms.size();
        ModuleExp mod = info.setupModuleExp();
        Vector declPairs = new Vector();
        for (Declaration fdecl = mod.firstDecl(); fdecl != null; fdecl = fdecl.nextDecl()) {
            if (!fdecl.isPrivate()) {
                Symbol aname = (Symbol) fdecl.getSymbol();
                if (renamer != null) {
                    try {
                        obj = renamer.apply1(aname);
                    } catch (Throwable ex4) {
                        obj = ex4;
                    }
                    if (obj != null) {
                        if (!(obj instanceof Symbol)) {
                            tr.error('e', "internal error - import name mapper returned non-symbol: " + obj.getClass().getName());
                        } else {
                            aname = (Symbol) obj;
                        }
                    }
                }
                boolean isStatic = fdecl.getFlag(PlaybackStateCompat.ACTION_PLAY_FROM_SEARCH);
                if (!isStatic && decl == null) {
                    String iname = tname.replace('.', '$') + "$instance";
                    decl = new Declaration(SimpleSymbol.valueOf(iname), type);
                    decl.setPrivate(true);
                    decl.setFlag(1073758208L);
                    defs.addDeclaration(decl);
                    decl.noteValue(dofind);
                    SetExp sexp = new SetExp(decl, dofind);
                    sexp.setLine(tr);
                    sexp.setDefining(true);
                    forms.addElement(sexp);
                    formsStart = forms.size();
                    decl.setFlag(536870912L);
                    if (isRunnable) {
                        decl.setSimple(false);
                    }
                    decl.setFlag(8192L);
                }
                if (fdecl.field != null) {
                    String fname = fdecl.field.getName();
                    if (fname.equals("$instance")) {
                        instanceField = fdecl.field;
                    }
                }
                boolean isImportedInstance = fdecl.field != null && fdecl.field.getName().endsWith("$instance");
                Declaration old = defs.lookup(aname, language2, language2.getNamespaceOf(fdecl));
                if (isImportedInstance) {
                    if (old == null) {
                        adecl = defs.addDeclaration(aname);
                        adecl.setFlag(1073758208L);
                        adecl.setType(fdecl.getType());
                        adecl.setFlag(8192L);
                        adecl.setLocation(tr);
                        ReferenceExp fref = new ReferenceExp(fdecl);
                        fref.setContextDecl(decl);
                        if (!isImportedInstance) {
                            fref.setDontDereference(true);
                            if (!sharedModule) {
                                adecl.setPrivate(true);
                            }
                        }
                        adecl.setFlag(16384L);
                        if (fdecl.getFlag(32768L)) {
                            adecl.setFlag(32768L);
                        }
                        if (fdecl.isProcedureDecl()) {
                            adecl.setProcedureDecl(true);
                        }
                        if (isStatic) {
                            adecl.setFlag(PlaybackStateCompat.ACTION_PLAY_FROM_SEARCH);
                        }
                        SetExp sexp2 = new SetExp(adecl, (Expression) fref);
                        adecl.setFlag(536870912L);
                        sexp2.setDefining(true);
                        if (!isImportedInstance) {
                            forms.insertElementAt(sexp2, formsStart);
                            formsStart++;
                        } else {
                            forms.addElement(sexp2);
                        }
                        declPairs.add(adecl);
                        declPairs.add(fdecl);
                        adecl.noteValue(fref);
                        adecl.setFlag(131072L);
                        tr.push(adecl);
                    }
                } else if (old == null || old.getFlag(512L) || Declaration.followAliases(old) != Declaration.followAliases(fdecl)) {
                    if (old != null && old.getFlag(66048L)) {
                        old.setFlag(false, 66048L);
                        adecl = old;
                    } else {
                        adecl = defs.addDeclaration(aname);
                        if (old != null) {
                            ScopeExp.duplicateDeclarationError(old, adecl, tr);
                        }
                    }
                    adecl.setAlias(true);
                    adecl.setIndirectBinding(true);
                    adecl.setLocation(tr);
                    ReferenceExp fref2 = new ReferenceExp(fdecl);
                    fref2.setContextDecl(decl);
                    if (!isImportedInstance) {
                    }
                    adecl.setFlag(16384L);
                    if (fdecl.getFlag(32768L)) {
                    }
                    if (fdecl.isProcedureDecl()) {
                    }
                    if (isStatic) {
                    }
                    SetExp sexp22 = new SetExp(adecl, (Expression) fref2);
                    adecl.setFlag(536870912L);
                    sexp22.setDefining(true);
                    if (!isImportedInstance) {
                    }
                    declPairs.add(adecl);
                    declPairs.add(fdecl);
                    adecl.noteValue(fref2);
                    adecl.setFlag(131072L);
                    tr.push(adecl);
                }
            }
        }
        int ndecls = declPairs.size();
        for (int i = 0; i < ndecls; i += 2) {
            Declaration adecl2 = (Declaration) declPairs.elementAt(i);
            Declaration fdecl2 = (Declaration) declPairs.elementAt(i + 1);
            Expression fval = fdecl2.getValue();
            if (fdecl2.isIndirectBinding() && (fval instanceof ReferenceExp)) {
                ReferenceExp aref = (ReferenceExp) adecl2.getValue();
                Declaration xdecl = ((ReferenceExp) fval).getBinding();
                aref.setBinding(xdecl);
                if (xdecl.needsContext()) {
                    String iname2 = xdecl.field.getDeclaringClass().getName().replace('.', '$') + "$instance";
                    Declaration cdecl = defs.lookup(SimpleSymbol.valueOf(iname2));
                    cdecl.setFlag(PlaybackStateCompat.ACTION_PLAY_FROM_MEDIA_ID);
                    aref.setContextDecl(cdecl);
                }
            }
        }
        if (isRunnable) {
            Method run = Compilation.typeRunnable.getDeclaredMethod("run", 0);
            if (decl != null) {
                dofind = new ReferenceExp(decl);
            } else if (instanceField != null) {
                Expression[] args2 = {new QuoteExp(type), new QuoteExp("$instance")};
                dofind = new ApplyExp(SlotGet.staticField, args2);
            }
            ApplyExp applyExp = new ApplyExp(run, dofind);
            applyExp.setLine(tr);
            forms.addElement(applyExp);
        }
        return true;
    }

    @Override // kawa.lang.Syntax
    public Expression rewriteForm(Pair form, Translator tr) {
        return null;
    }
}
