package org.acra;

import android.content.Context;
import android.support.v7.internal.widget.ActivityChooserView;
import gnu.kawa.servlet.HttpRequestContext;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.util.Map;
import org.acra.collector.CrashReportData;
/* loaded from: classes.dex */
final class CrashReportPersister {
    private static final int CONTINUE = 3;
    private static final int IGNORE = 5;
    private static final int KEY_DONE = 4;
    private static final String LINE_SEPARATOR = "\n";
    private static final int NONE = 0;
    private static final int SLASH = 1;
    private static final int UNICODE = 2;
    private final Context context;

    /* JADX INFO: Access modifiers changed from: package-private */
    public CrashReportPersister(Context context) {
        this.context = context;
    }

    public CrashReportData load(String fileName) throws IOException {
        CrashReportData load;
        FileInputStream in = this.context.openFileInput(fileName);
        if (in == null) {
            throw new IllegalArgumentException("Invalid crash report fileName : " + fileName);
        }
        try {
            BufferedInputStream bis = new BufferedInputStream(in, 8192);
            bis.mark(ActivityChooserView.ActivityChooserViewAdapter.MAX_ACTIVITY_COUNT_UNLIMITED);
            boolean isEbcdic = isEbcdic(bis);
            bis.reset();
            if (!isEbcdic) {
                load = load(new InputStreamReader(bis, "ISO8859-1"));
            } else {
                load = load(new InputStreamReader(bis));
            }
            return load;
        } finally {
            in.close();
        }
    }

    public void store(CrashReportData crashData, String fileName) throws IOException {
        OutputStream out = this.context.openFileOutput(fileName, 0);
        try {
            StringBuilder buffer = new StringBuilder((int) HttpRequestContext.HTTP_OK);
            OutputStreamWriter writer = new OutputStreamWriter(out, "ISO8859_1");
            for (Map.Entry<ReportField, String> entry : crashData.entrySet()) {
                String key = entry.getKey().toString();
                dumpString(buffer, key, true);
                buffer.append('=');
                dumpString(buffer, entry.getValue(), false);
                buffer.append(LINE_SEPARATOR);
                writer.write(buffer.toString());
                buffer.setLength(0);
            }
            writer.flush();
        } finally {
            out.close();
        }
    }

    private boolean isEbcdic(BufferedInputStream in) throws IOException {
        byte b;
        do {
            b = (byte) in.read();
            if (b == -1 || b == 35 || b == 10 || b == 61) {
                return false;
            }
        } while (b != 21);
        return true;
    }

    /* JADX WARN: Removed duplicated region for block: B:135:0x0172 A[SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:42:0x00b1  */
    /* JADX WARN: Removed duplicated region for block: B:55:0x00de  */
    /* JADX WARN: Removed duplicated region for block: B:84:0x0130  */
    /* JADX WARN: Removed duplicated region for block: B:96:0x0178  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private synchronized CrashReportData load(Reader reader) throws IOException {
        CrashReportData crashData;
        char nextChar;
        int mode = 0;
        int unicode = 0;
        int count = 0;
        char[] buf = new char[40];
        int keyLength = -1;
        boolean firstChar = true;
        crashData = new CrashReportData();
        BufferedReader br = new BufferedReader(reader, 8192);
        int offset = 0;
        while (true) {
            int intVal = br.read();
            if (intVal != -1) {
                char nextChar2 = (char) intVal;
                if (offset == buf.length) {
                    char[] newBuf = new char[buf.length * 2];
                    System.arraycopy(buf, 0, newBuf, 0, offset);
                    buf = newBuf;
                }
                if (mode == 2) {
                    int digit = Character.digit(nextChar2, 16);
                    if (digit >= 0) {
                        unicode = (unicode << 4) + digit;
                        count++;
                        if (count >= 4) {
                        }
                    } else if (count <= 4) {
                        throw new IllegalArgumentException("luni.09");
                    }
                    mode = 0;
                    int offset2 = offset + 1;
                    buf[offset] = (char) unicode;
                    if (nextChar2 == '\n' || nextChar2 == 133) {
                        offset = offset2;
                    } else {
                        offset = offset2;
                    }
                }
                if (mode == 1) {
                    mode = 0;
                    switch (nextChar2) {
                        case '\n':
                        case 133:
                            mode = 5;
                            break;
                        case '\r':
                            mode = 3;
                            break;
                        case 'b':
                            nextChar2 = '\b';
                            firstChar = false;
                            if (mode == 4) {
                                keyLength = offset;
                                mode = 0;
                            }
                            buf[offset] = nextChar2;
                            offset++;
                            break;
                        case 'f':
                            nextChar2 = '\f';
                            firstChar = false;
                            if (mode == 4) {
                            }
                            buf[offset] = nextChar2;
                            offset++;
                            break;
                        case 'n':
                            nextChar2 = '\n';
                            firstChar = false;
                            if (mode == 4) {
                            }
                            buf[offset] = nextChar2;
                            offset++;
                            break;
                        case 'r':
                            nextChar2 = '\r';
                            firstChar = false;
                            if (mode == 4) {
                            }
                            buf[offset] = nextChar2;
                            offset++;
                            break;
                        case 't':
                            nextChar2 = '\t';
                            firstChar = false;
                            if (mode == 4) {
                            }
                            buf[offset] = nextChar2;
                            offset++;
                            break;
                        case 'u':
                            mode = 2;
                            count = 0;
                            unicode = 0;
                            break;
                        default:
                            firstChar = false;
                            if (mode == 4) {
                            }
                            buf[offset] = nextChar2;
                            offset++;
                            break;
                    }
                } else {
                    switch (nextChar2) {
                        case '\n':
                            if (mode == 3) {
                                mode = 5;
                                break;
                            } else {
                                mode = 0;
                                firstChar = true;
                                if (offset <= 0 || (offset == 0 && keyLength == 0)) {
                                    if (keyLength == -1) {
                                        keyLength = offset;
                                    }
                                    String temp = new String(buf, 0, offset);
                                    crashData.put((CrashReportData) Enum.valueOf(ReportField.class, temp.substring(0, keyLength)), (Enum) temp.substring(keyLength));
                                }
                                keyLength = -1;
                                offset = 0;
                                break;
                            }
                        case '\r':
                        case 133:
                            mode = 0;
                            firstChar = true;
                            if (offset <= 0) {
                            }
                            if (keyLength == -1) {
                            }
                            String temp2 = new String(buf, 0, offset);
                            crashData.put((CrashReportData) Enum.valueOf(ReportField.class, temp2.substring(0, keyLength)), (Enum) temp2.substring(keyLength));
                            keyLength = -1;
                            offset = 0;
                            break;
                        case '!':
                        case '#':
                            if (!firstChar) {
                                if (Character.isWhitespace(nextChar2)) {
                                    if (mode == 3) {
                                        mode = 5;
                                    }
                                    if (offset != 0 && offset != keyLength && mode != 5) {
                                        if (keyLength == -1) {
                                            mode = 4;
                                            break;
                                        }
                                    }
                                }
                                if (mode != 5 || mode == 3) {
                                    mode = 0;
                                }
                                firstChar = false;
                                if (mode == 4) {
                                }
                                buf[offset] = nextChar2;
                                offset++;
                                break;
                            } else {
                                do {
                                    int intVal2 = br.read();
                                    if (intVal2 != -1 && (nextChar = (char) intVal2) != '\r' && nextChar != '\n') {
                                    }
                                } while (nextChar != 133);
                                break;
                            }
                            break;
                        case ':':
                        case '=':
                            if (keyLength == -1) {
                                mode = 0;
                                keyLength = offset;
                                break;
                            } else {
                                if (Character.isWhitespace(nextChar2)) {
                                }
                                if (mode != 5) {
                                    break;
                                }
                                mode = 0;
                                firstChar = false;
                                if (mode == 4) {
                                }
                                buf[offset] = nextChar2;
                                offset++;
                                break;
                            }
                            break;
                        case '\\':
                            if (mode == 4) {
                                keyLength = offset;
                            }
                            mode = 1;
                            break;
                        default:
                            if (Character.isWhitespace(nextChar2)) {
                            }
                            if (mode != 5) {
                            }
                            mode = 0;
                            firstChar = false;
                            if (mode == 4) {
                            }
                            buf[offset] = nextChar2;
                            offset++;
                            break;
                    }
                }
            } else if (mode == 2 && count <= 4) {
                throw new IllegalArgumentException("luni.08");
            } else {
                if (keyLength == -1 && offset > 0) {
                    keyLength = offset;
                }
                if (keyLength >= 0) {
                    String temp3 = new String(buf, 0, offset);
                    ReportField key = (ReportField) Enum.valueOf(ReportField.class, temp3.substring(0, keyLength));
                    String value = temp3.substring(keyLength);
                    if (mode == 1) {
                        value = value + "\u0000";
                    }
                    crashData.put((CrashReportData) key, (ReportField) value);
                }
            }
        }
        return crashData;
    }

    private void dumpString(StringBuilder buffer, String string, boolean key) {
        int i = 0;
        if (!key && 0 < string.length() && string.charAt(0) == ' ') {
            buffer.append("\\ ");
            i = 0 + 1;
        }
        while (i < string.length()) {
            char ch = string.charAt(i);
            switch (ch) {
                case '\t':
                    buffer.append("\\t");
                    break;
                case '\n':
                    buffer.append("\\n");
                    break;
                case 11:
                default:
                    if ("\\#!=:".indexOf(ch) >= 0 || (key && ch == ' ')) {
                        buffer.append('\\');
                    }
                    if (ch >= ' ' && ch <= '~') {
                        buffer.append(ch);
                        break;
                    } else {
                        String hex = Integer.toHexString(ch);
                        buffer.append("\\u");
                        for (int j = 0; j < 4 - hex.length(); j++) {
                            buffer.append("0");
                        }
                        buffer.append(hex);
                        break;
                    }
                    break;
                case '\f':
                    buffer.append("\\f");
                    break;
                case '\r':
                    buffer.append("\\r");
                    break;
            }
            i++;
        }
    }
}
